
-- Sun, Moon and Planets Astro Plotter  MrPeachy5 Sean aka foodoo January 2019 --

require 'cairo'

local w = {true}

local lcairo_scale                       = cairo_scale
local lcairo_create                      = cairo_create
local lcairo_xlib_surface_create         = cairo_xlib_surface_create
local lconky_parse                       = conky_parse
local lcairo_destroy                     = cairo_destroy
local lcairo_surface_destroy             = cairo_surface_destroy
local lcairo_rel_line_to                 = cairo_rel_line_to
local lcollectgarbage                    = collectgarbage
local lpairs                             = pairs
local lcairo_save                        = cairo_save
local lcairo_restore                     = cairo_restore
local lcairo_fill                        = cairo_fill
local lcairo_set_source_rgba             = cairo_set_source_rgba
local lcairo_set_line_width              = cairo_set_line_width
local lcairo_arc                         = cairo_arc
local lcairo_pattern_add_color_stop_rgba = cairo_pattern_add_color_stop_rgba
local lcairo_pattern_create_radial       = cairo_pattern_create_radial
local lcairo_pattern_create_linear       = cairo_pattern_create_linear
local lcairo_set_source                  = cairo_set_source
local lcairo_stroke                      = cairo_stroke
local lcairo_pattern_destroy             = cairo_pattern_destroy
local lcairo_set_line_cap                = cairo_set_line_cap
local lcairo_move_to                     = cairo_move_to
local lcairo_line_to                     = cairo_line_to
local lcairo_set_font_size               = cairo_set_font_size
local lcairo_select_font_face            = cairo_select_font_face
local lcairo_show_text                   = cairo_show_text
local lcairo_text_extents                = cairo_text_extents
local lcairo_stroke_preserve             = cairo_stroke_preserve
local lcairo_rectangle                   = cairo_rectangle
local lcairo_clip                        = cairo_clip

local PI        = 3.14159265358979323846
local twoPI     = 3.14159265358979323846 * 2
local TORADIANS = 3.14159265358979323846/180
local TODEGREES = 180/3.14159265358979323846

local PHI = 1.6180339887499

local function toRadians(x) return x * (PI / 180) end
local function toDegrees(x) return x * (180 / PI) end

local MT     = require('math')
local sin    = MT.sin
local cos    = MT.cos
local tan    = MT.tan
local atan2  = MT.atan2
local asin   = MT.asin
local acos   = MT.acos
local abs    = MT.abs
local sqrt   = MT.sqrt
local lfloor = MT.floor
local lceil  = MT.ceil
local rad    = MT.rad
local tonum  = tonumber

local function sind(x) return sin(toRadians(x)) end
local function cosd(x) return cos(toRadians(x)) end
local function tand(x) return tan(toRadians(x)) end

local function atand(x)    return (toRadians(atan(x)))    end
local function asind(x)    return (toRadians(asin(x)))    end
local function acosd(x)    return (toRadians(acos(x)))    end

local function atan2d(y,x) return (toRadians(atan2(y,x))) end

local TB = require('table')
local tconcat   = TB.concat

-- colour reference en.wikipedia.org/wiki/Web_colors
local DeepSkyBlue    = 0x00BFFF
local RoyalBlue      = 0x355aAf
local SkyBlue        = 0x87CEEB
local LightSteelBlue = 0xb0c4de
local Peru           = 0xCD853F
local LightGrey      = 0xf1f1f1
local LightGrey2     = 0xf5f5f5
local Grey           = 0x555555
local DarkGrey       = 0x333333
local DarkSeaGreen   = 0x8fbc8f
local Red            = 0xff0000
local Tomato         = 0xff6347
local Magenta        = 0xff00ff
local Lime           = 0x00ff00
local Cyan           = 0x00ffff
local DarkCyan       = 0x007fff
local Blue           = 0x0000ff
local Yellow         = 0xffff00
local White          = 0xffffff
local OffWhite       = 0xbbbbcc
local Black          = 0x000000
local Orange         = 0xFF3300
local Orange3        = 0xFF5500
local SaddleBrown    = 0x8B4513
local Oldgold        = 0xCFB53B
local DarkOlive      = 0x044331
local Green          = 0x008000

local FC = require("luafunc")
local wxsplit          = FC.wxsplit
local background       = FC.background
local to_rgba          = FC.to_rgba
local to_rgba_contrast = FC.to_rgba_contrast
local set_font         = FC.set_font
local make_txt         = FC.make_txt
local make_scale_txt2  = FC.make_scale_txt2
local choose_indicator = FC.choose_indicator
local get_points       = FC.get_points
local put_image        = FC.put_image
local ExecInfo         = FC.ExecInfo
local split_newlines   = FC.split_newlines
local FileExists       = FC.FileExists
local hrsToDec         = FC.hoursToDecimal

local ST = require('string')
-- local supper    = ST.upper
-- local lfind     = ST.find
-- local lsub      = ST.sub
-- local sprintf   = ST.format
local llen      = ST.len
local lmatch    = ST.match
local lgsub     = ST.gsub

local OS       = require('os')
local ldate    = OS.date
local ltime    = OS.time
local lexecute = OS.execute


local clockRadius = 147	-- Radius of Clock

local magellan
local mag = {}
local tzoneText = ldate("%Z")
local tzoneNum  = hrsToDec(ldate("%z"))
local time = {true}
-- local updates  = tonum(lconky_parse("${updates}"))

-- local windowHt = tonum(conky_window.height)

local t63, t36 = 0, 0

local updates, clock24

local function labelMaker(cr,px,py,text,fontSize,te)
    local numCharacters = llen(text)
    -- local numCharacters = 8
    -- print(numCharacters)
    lcairo_set_source_rgba(cr, 0,0,0,0.75)
    local recWidth = fontSize * (numCharacters - math.log(numCharacters-1.1))
    lcairo_rectangle(cr, px-3, py+4, recWidth, -(te.height + 6))
    lcairo_fill(cr)
end

local function position_text(cr,t,text_x,text_y,pos,te)
    -- adjust text location
    -- get text area (x_bearing,y_bearing,width,height,x_advance,y_advance)
    -- local te = cairo_text_extents_t:create()
    -- lcairo_text_extents(cr,t,te)
    local xtext = text_x
    local ytext = text_y

    if ( pos == 1 ) then    -- centre text
        xtext = text_x - te.width / 2
        ytext = text_y + te.height / 2
    elseif ( pos == 2 ) then    -- horizontal align
        xtext = text_x - te.width / 2
    elseif ( pos == 3 ) then    -- vertical align
        ytext = text_y + te.height / 2
    elseif ( pos == 4 ) then    -- set right edge of text to pos
        xtext = text_x - te.width
    end

    return xtext,ytext
end

local function printText(cr,text,xT,yT,posT,fontSize,ff,box,font)
    local te = cairo_text_extents_t:create()
    lcairo_text_extents(cr,text,te)

    lcairo_set_font_size (cr,fontSize)
    local xt, yt
    if ( posT == 0 ) then
        xt = xT
        yt = yT
    else
        xt,yt = position_text(cr,text,xT,yT,posT,te)
    end

    font = font or "DejaVu Sans"
    if not ff then
        lcairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD)
    else
        lcairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL)
    end


    if box then
        lcairo_save(cr)
        labelMaker(cr,xt,yt,text,fontSize,te)
        lcairo_restore(cr)
    end

    lcairo_move_to (cr,xt,yt)
    lcairo_show_text (cr,text)
    lcairo_stroke (cr)
end



local function draw_sun_ring(p)

    local cr = p.cr
    local px = p.xSun or 250
    local py = p.ySun or 225

    local start_angle = 90
    local end_angle   = 360

    lcairo_arc(cr, px, py, p.radiusSun+7.8, TORADIANS, 0 )
    lcairo_set_source_rgba(cr,to_rgba({RoyalBlue,1}))
    -- lcairo_clip(cr)
    lcairo_fill(cr)

    local border_pat = lcairo_pattern_create_linear(px,py - p.radiusSun * 1.25,px,py + p.radiusSun * 1.25)

    lcairo_pattern_add_color_stop_rgba(border_pat,0,3,1,0.5,1)
    lcairo_pattern_add_color_stop_rgba(border_pat,0.35,0.9,0.9,0.2,1)
    lcairo_pattern_add_color_stop_rgba(border_pat,0.6,0.9,0.2,0,1)
    lcairo_pattern_add_color_stop_rgba(border_pat,0.8,0,0.1,1,1)

    lcairo_set_source(cr,border_pat)

    -- draw ring, starting at (midnight/00hrs)
    lcairo_arc(cr, px, py, p.radiusSun, toRadians(start_angle), toRadians(start_angle + end_angle) )

    lcairo_set_line_width(cr,p.radiusSun * 0.08)
    lcairo_stroke(cr)
    lcairo_pattern_destroy (border_pat)

end


local function rimMarker(p)

    local function isValue(val)
        if not val then return 0 end
        return 15 * hrsToDec(val)
    end

    local cr = p.p.cr
    lcairo_save(cr)

    local px          = p.p.xSun
    local py          = p.p.ySun
    local fontSize    = p.p.labelFontSize
    local llabels     = p.p.llabels
    local radiusA     = p.radiusA or 1.05
    local radiusB     = p.radiusB or 0.95
    local markColour  = p.markColour  or p.labelColour
    local labelColour = p.labelColour or p.markColour
    local startAngle  = p.startAngle or 0
    local radiusSun   = p.p.radiusSun
    local corrY = 0
    local corrX = 0

    local event       = isValue(p.event)
    local labelType   = p.labelType
    local labelText   = p.labelText
    local labelTime   = (p.event or '')
    local labelComb   = labelType .. '-' .. labelTime


    local valX,  valY  = get_points(startAngle, event, 0, radiusSun * radiusA, 0, px, py)
    local valXc, valYc = get_points(startAngle, event, 0, radiusSun * radiusB, 0, px, py)

    lcairo_set_source_rgba(cr,to_rgba({markColour,1}))
    lcairo_move_to(cr, valX,  valY)
    lcairo_line_to(cr, valXc, valYc)
    lcairo_stroke(cr)

    local arrHarry  = {MR=1,MT=1,MS=1}
    local arrHarryRise = {ST=1,SR=1,SS=1,CS=1,CR=1,NR=1,NS=1,AS=1,AR=1}

    local ftable = {
        { label = 1, event = 270, xcorr =   5, ycorr =  10, },
        { label = 1, event = 180, xcorr =   5, ycorr =  -5, },
        { label = 1, event =  90, xcorr = -60, ycorr =  -5, },
        { label = 1, event =   0, xcorr = -60, ycorr =  10, },
        { label = 2, event = 270, xcorr =   5, ycorr =  10, },
        { label = 2, event = 180, xcorr =   5, ycorr =   0, },
        { label = 2, event =  90, xcorr = -20, ycorr =  -5, },
        { label = 2, event =   0, xcorr = -20, ycorr = -10, },
        { label = 3, event = 270, xcorr =   5, ycorr =  10, },
        { label = 3, event = 180, xcorr =   5, ycorr =  -5, },
        { label = 3, event =  90, xcorr = -20, ycorr =  -5, },
        { label = 3, event =   0, xcorr = -20, ycorr =  10, },
    }

    local labelHarry
    if llabels < 4 then
        if llabels == 2 then
            if arrHarry[labelType] then
                labelHarry = labelType
                if event > 270 then
                    corrX = 5
                    corrY = 10
                elseif event > 180 then
                    corrX = 5
                    corrY = 0
                elseif event == 180 then
                    corrX = 0
                    corrY = -5
                elseif event > 90 then
                    corrX = -20
                    corrY = -5
                elseif event > 0 then
                    corrX = -20
                    corrY = 10
                else
                    corrX = 0
                    corrY = 10
                end
            else
                if arrHarryRise[labelType] then
                    if event > 270 then
                        corrX = 5
                        corrY = 10
                    elseif event > 180 then
                        corrX = 5
                        corrY = -5
                    elseif event == 180 then
                        corrX = 0
                        corrY = -5
                    elseif event > 90 then
                        corrX = -60
                        corrY = -5
                    elseif event > 0 then
                        corrX =  -60
                        corrY =  10
                    else
                        corrX = 0
                        corrY = 10
                    end
                end
                labelHarry = labelComb
            end
        else
            labelHarry = labelType
        end

        if llabels == 1 then
            labelHarry = labelComb
            if event > 270 then
                corrX = 5
                corrY = 10
            elseif event > 180 then
                corrX = 5
                corrY = -5
            elseif event == 180 then
                corrX = 0
                corrY = -5
            elseif event > 90 then
                corrX = -60
                corrY = -5
            elseif event > 0 then
                corrX = -60
                corrY = 10
            else
                corrX = 0
                corrY = 10
            end
        end
        if llabels == 3 then
            labelHarry = labelType
            if event > 270 then
                corrX = 5
                corrY = 10
            elseif event > 180 then
                corrX = 5
                corrY = -5
            elseif event == 180 then
                corrX = 0
                corrY = -5
            elseif event > 90 then
                corrX = -20
                corrY = -5
            elseif event > 0 then
                corrX = -20
                corrY = 10
            else
                corrX = 0
                corrY = 10
            end
        end

        -- print(labelHarry, corrX,corrY)
        -- display time
        lcairo_set_source_rgba (cr,to_rgba({labelColour,1}))
        printText(cr, labelHarry, valX + (corrX or 0), valY + (corrY or 0), 0, fontSize,false,true)
    end

    if llabels <= 2 then
        lcairo_save(cr)
        lcairo_set_source_rgba (cr,to_rgba({labelColour,1}))
        local ltext = tconcat{
            labelTime,' - ',
            labelType,' - ',
            labelText
        }
        printText(cr,
            ltext,
            px - 100,
            py + 210 + (p.count * (5 + fontSize)),
            0,
            fontSize,
            false,
            false
        )
        lcairo_restore(cr)
    end


end

local function markerRST(p)

    local cr = p.cr

    lcairo_set_line_width(cr, 3)
    lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND)

    local marks = {
        {  -- draw sun transit mark
            count       = 1,
            event       = mag.sun_mer_trans,
            labelType   = 'ST',
            labelText   = 'Sun Transit',
            markColour  = Orange3,
            -- corrX       = 0,
            -- corrY       = 0,
            p           = p
        },
        ------ Rise
        {   -- draw sunrise mark
            count       = 2,
            event       = mag.sun_rise,
            labelType   = 'SR',
            labelText   = 'Sun Rise',
            markColour  = Orange3,
            -- corrX       = -20,
            -- corrY       = 10,
            p           = p
        },
        {  -- civil twilight rise
            count       = 3,
            event       = mag.civ_twilt_R,
            labelType   = 'CR',
            labelText   = 'Civil Twilight Rise',
            markColour  = Lime,
            -- corrX       = -20,
            -- corrY       = 10,
            p           = p
        },
        {  -- nautical twilight rise
            count       = 4,
            event       = mag.nau_twilt_R,
            labelType   = 'NR',
            labelText   = 'Nautical Twilight Rise',
            markColour  = Lime,
            -- corrX       = -20,
            -- corrY       = 10,
            p           = p
        },
        {  -- astronomical twilight rise
            count       = 5,
            event       = mag.ast_twilt_R,
            labelType   = 'AR',
            labelText   = 'Astronomical Twilight Rise',
            markColour  = Lime,
            -- corrX       = -20,
            -- corrY       = 10,
            p           = p
        },
        ------ Set
        {   -- draw sunset mark
            count       = 6,
            event       = mag.sun_set,
            labelType   = 'SS',
            labelText   = 'Sun Set',
            markColour  = Orange3,
            -- corrX       = 5,
            -- corrY       = 10,
            p           = p
        },
        {  -- civil twilight set
            count       = 7,
            event       = mag.civ_twilt_S,
            labelType   = 'CS',
            labelText   = 'Civil Twilight Set',
            markColour  = Lime,
            -- corrX       = 5,
            -- corrY       = 10,
            p           = p
        },
        {  -- nautical twilight set
            count       = 8,
            event       = mag.nau_twilt_S,
            labelType   = 'NS',
            labelText   = 'Nautical Twilight Set',
            markColour  = Lime,
            -- corrX       = 5,
            -- corrY       = 10,
            p           = p
        },
        {  -- astronomical twilight set
            count       = 9,
            event       = mag.ast_twilt_S,
            labelType   = 'AS',
            labelText   = 'Astronomical Twilight Set',
            markColour  = Lime,
            -- corrX       = 5,
            -- corrY       = 10,
            p           = p
        },
        ----- Moon
        {  -- draw moon Trasit mark
            count       = 10,
            type        = 1,
            event       = mag.moon_mer_trans,
            labelType   = 'MT',
            labelText   = 'Moon Transit',
            markColour  = OffWhite,
            -- corrX       = 0,
            -- corrY       = 0,
            p           = p
        },
        {  -- draw moon rise mark
            count       = 11,
            type        = 1,
            event       = mag.moon_rise,
            labelType   = 'MR',
            labelText   = 'Moon Rise',
            markColour  = OffWhite,
            -- corrX       = -15,
            -- corrY       = 5,
            p           = p
        },
        {   -- draw moon set mark
            count       = 12,
            type        = 1,
            event       = mag.moon_set,
            labelType   = 'MS',
            labelText   = 'Moon Set',
            markColour  = OffWhite,
            -- corrX       = 0,
            -- corrY       = 0,
            p           = p
        },
          -- draw TEST mark
        -- {  -- draw TEST mark
        --     type        = 9,
        --     count       = 13,
        --     event       = (tonum(p.csec) or 0) / 15 * 6,
        --     -- eventST     = (tonum(mag.sun_mer_trans_T) or 0) * 15,
        --     -- eventSR     = (tonum(mag.sun_rise_T) or 0) * 15,
        --     -- eventAR     = (hrsToDec(mag.ast_twilt_R) or 0) * 15,
        --     -- eventSS     = (tonum(mag.sun_set_T) or 0) * 15,
        --     -- eventAS     = (hrsToDec(mag.ast_twilt_S) or 0) * 15,
        --     label       = 'Test-0:'..(p.csec),
        --     labelType   = 'TT',
        --     labelText   = 'Test Set',
        --     markColour  = OffWhite,
        --     corrX       = -65,
        --     corrY       = 10,
        --     p           = p
        -- },
    }

    for i in lpairs(marks) do
        rimMarker(marks[i])
    end

end


local function drawMarksNumbers(p)
    local cr = p.cr
    local r = p.radiusW
    local px = p.xSun or 250
    local py = p.ySun or 225
    local fontSize = p.fontSize

    local inner = r - 3
    local outer = r + 3

    if px == 0 then px = 250 end
    if py == 0 then py = 225 end

    -- clock24 = 12
    local startV, scaleF = 0,0
    for i = 0,360 do
        local x0,y0,endx,endy,degVal,txtNum

        degVal = 0

        lcairo_set_line_width(cr, 1)

        if i % 15 == 0 then
            if i-15 >= 0 then
                --- numbers
                if clock24 == 12 then
                    if i % 30 == 0 then
                        txtNum = lfloor(i / 30)
                        startV = 180; scaleF = 0
                    end
                else
                    txtNum = lfloor((i-15) / 15)
                    startV = 0; scaleF = 15
                end
                x0, y0 = get_points(
                    tonum(startV),
                    i - (tonum(scaleF) or 0),
                    0,
                    inner - 14,
                    0,
                    px,
                    py
                )
                lcairo_set_source_rgba(cr,to_rgba({Yellow,1}))
                printText(cr,txtNum,x0,y0,1,fontSize,false)
            end
            --- red marks
            degVal = i
            lcairo_set_source_rgba(cr,to_rgba({Red,1}))
            lcairo_set_line_width(cr, 3)
        else
            degVal = i * 2.5
            -- degVal = i * 3
            if degVal % 15 == 0 then
                --- red marks
                lcairo_set_source_rgba(cr,to_rgba({Red,1}))
                lcairo_set_line_width(cr, 3)
            else
                --- white marks
                lcairo_set_source_rgba(cr,to_rgba({White,1}))
                lcairo_set_line_width(cr, 1)
            end
        end

        x0,   y0   = get_points(0, degVal, 0, inner-28, 0, px, py)
        endx, endy = get_points(0, degVal, 0, outer-28, 0, px, py)

        lcairo_move_to(cr, x0, y0  )
        lcairo_line_to(cr, endx,endy)
        lcairo_stroke(cr)

        -- seconds marks
        degVal = i * 6
        x0,   y0   = get_points(0, degVal, 0, inner - 5, 0, px, py)
        endx, endy = get_points(0, degVal, 0, outer - 5, 0, px, py)

        if degVal % 30 == 0 then
            lcairo_set_source_rgba(cr,to_rgba({Red,1}))
            lcairo_set_line_width(cr, 3)
        else
            lcairo_set_source_rgba(cr,to_rgba({DarkSeaGreen,1}))
            lcairo_set_line_width(cr, 1)
        end
        lcairo_move_to(cr, x0,  y0  )
        lcairo_line_to(cr, endx,endy)
        lcairo_stroke(cr)

    end
end


local function drawLatLong(p)
    local cr          = p.cr
    local px          = p.xSun -- 250
    local py          = p.ySun -- 225
    local colourLines = 0x888d8f
    local colourRings = Peru
    local elipFactor  = 6
    local radiusHorizon = 160 - 0 --50
    local latHorizon = 130 - 0 --80

    -- Draw Circle Matrix ---------------------------------------
    local lines = {
        {    -- rings around dial numbers outer
            fill        = 21,
            colour      = colourRings,
            lineWidth   = 3,
            ctrx        = px,
            ctry        = py,
            radius      = 180,
        },
        {    -- rings around dial numbers outer
            fill        = 24,
            colour      = colourRings,
            lineWidth   = 3,
            ctrx        = px,
            ctry        = py,
            radius      = 147,
        },
        {    -- clip ring
            fill        = 23,
            colour      = colourRings,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py,
            radius      = 147,
        },
        -- longitudes
        {   -- long 8W / 4E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - 61,
            ctry        = 147,
            radius      = 140,
        },
        {   -- long 4W / 8E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px + 61,
            ctry        = 147,
            radius      = 140,
        },
        {   -- long 5W / 7E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px + 30,
            ctry        = 147,
            radius      = 130,
        },
        {   -- long 5E / 7W
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - 30,
            ctry        = 147,
            radius      = 130,
        },
        {   -- No 6 long e/w of obs long
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = 147,
            radius      = 127,
        },
        {   -- long 9W / 3E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - 106,
            ctry        = 147,
            radius      = 165,
        },
        {   -- long 3W / 9E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px + 106,
            ctry        = 147,
            radius      = 165,
        },
        {   -- long 2W / 10E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px + 180,
            ctry        = 147,
            radius      = 220,
        },
        {   -- long 10W / 2E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - 180,
            ctry        = 147,
            radius      = 220,
        },
        {   -- long 1W / 11E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px + 340,
            ctry        = 147,
            radius      = 360,
        },
        {   -- long 11W / 1E
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - 340,
            ctry        = 147,
            radius      = 360,
        },
        ---Testing here
        {   -- No 1 lat from Up
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py + (-78 - 225),
            radius      = 170,
        },
        {   -- No 2 lat from Up
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py + -140 - 225,
            radius      = 250,
        },
        {   -- No 3 lat from Up
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py + (-120 - 225),
            radius      = 250,
        },
        {   -- No 5 lat from Up
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py + 758 - 225,
            radius      = 600,
        },
        {   -- No 6 lat from Up
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py + 513 - 225,
            radius      = 340,
        },
        {   -- No 7 lat from Up
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px,
            ctry        = py + 382 - 225,
            radius      = 201,
        },
        -- -- Horizon
        {   -- horizon
            type        = 1,
            fill        = 2,
            colour      = colourRings,
            lineWidth   = 2,
            ctrx        = px - elipFactor,
            ctry        = py + latHorizon,
            radius      = radiusHorizon,
        },
        {   -- civil twilight
            type        = 1,
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - elipFactor,
            ctry        = py + latHorizon - 12,
            radius      = radiusHorizon - 20,
        },
        {   -- nautical twilight
            type        = 1,
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - elipFactor,
            ctry        = py + latHorizon - 24,
            radius      = radiusHorizon - 40,
        },
        {   --- astronomical twilight
            type        = 1,
            colour      = colourLines,
            lineWidth   = 1,
            ctrx        = px - elipFactor,
            ctry        = py + latHorizon - 36,
            radius      = radiusHorizon - 60,
        },
        {    --  capricorn
            fill        = 2, -- make this 2 for it to work.
            colour      = colourRings,
            lineWidth   = 2,
            ctrx        = px,
            ctry        = py,
            radius      = 64,
        },
        {   -- equator
            colour      = colourRings,
            lineWidth   = 2,
            ctrx        = px,
            ctry        = py,
            radius      = 96,
        },
        {    -- Horizontal line Perhaps observers latitude --
            colour    = colourLines,
            lineWidth = 1,
            mvx       = px + 130 - 250,
            mvy       = py + 142 - 225,
            rlx       = 54 + 186,
            rly       = 0,
        },
        {   -- virtical line perhaps observers longitude --
            colour    = colourLines,
            lineWidth = 1,
            mvx       = px,
            mvy       = py + clockRadius,
            rlx       = 0,
            rly       = -300,
        },
    }

    for i in lpairs(lines) do

        if lines[i].radius then
            lcairo_set_source_rgba(cr,to_rgba({lines[i].colour,1}))
            lcairo_set_line_width(cr,lines[i].lineWidth)
            lcairo_arc(
                cr,
                lines[i].ctrx,
                lines[i].ctry,
                lines[i].radius,
                0,
                toRadians(360)
            )

            if lines[i].fill == 1 then
                lcairo_set_source_rgba(cr,to_rgba({lines[i].colour,0.925}))
                lcairo_stroke_preserve(cr)
                lcairo_set_source_rgba(cr,to_rgba({DarkGrey,0.925}))
                lcairo_stroke(cr)
            elseif lines[i].fill == 2 then
                lcairo_set_source_rgba(cr,to_rgba({lines[i].colour,0.925}))
                lcairo_stroke_preserve(cr)
                lcairo_set_source_rgba(cr,to_rgba({DarkGrey,0.925}))
                lcairo_fill(cr)
            elseif lines[i].fill == 21 then
                lcairo_set_source_rgba(cr,to_rgba({lines[i].colour,0.925}))
                lcairo_stroke_preserve(cr)
                lcairo_set_source_rgba(cr,to_rgba({DarkOlive,0.9}))
                lcairo_fill(cr)
            elseif lines[i].fill == 23 then
                lcairo_clip(cr)
            elseif lines[i].fill == 24 then
                lcairo_set_source_rgba(cr,to_rgba({lines[i].colour,0.925}))
                lcairo_stroke_preserve(cr)
                lcairo_set_source_rgba(cr,to_rgba({RoyalBlue,0.75}))
                lcairo_fill(cr)
            end
            lcairo_stroke(cr)
        elseif lines[i].mvx then

            lcairo_set_source_rgba(cr,to_rgba({lines[i].colour,1}))
            lcairo_set_line_width(cr,lines[i].lineWidth)

            lcairo_move_to( cr, lines[i].mvx, lines[i].mvy )
            lcairo_rel_line_to( cr, lines[i].rlx, lines[i].rly )
            lcairo_stroke(cr)
        end

    end
end

local function showTimeHands(p)
    local cr = p.cr
    local xc = p.xSun
    local yc = p.ySun

    local startF, scaleF
    if clock24 == 24 then
        startF = 180; scaleF = 15
    else
        startF = 0  ; scaleF = 30
    end

    lcairo_save(cr)
    local ciData = {
        {
            cr       = cr,
            px       = xc,
            py       = yc,
            startval = startF,
            scale    = p.chour * scaleF,
            rout1    = clockRadius * 1.01,
            ch_indi  = p.hourArrow or 7,
            cthick   = 4,
            length   = 1.375,
            chands   = 1,
            csize    = 2,
            icolour  = {p.hourColour,0.85},
        },
        {
            cr       = cr,
            px       = xc,
            py       = yc,
            startval = 0,
            scale    = p.cmin * 6,
            rout1    = clockRadius * 1.02,
            ch_indi  = p.minArrow or 7,
            cthick   = 4,
            length   = 1.20,
            chands   = 1,
            csize    = 2,
            icolour  = {p.minColour,0.85},
        },
        {
            cr       = cr,
            px       = xc,
            py       = yc,
            startval = 0,
            scale    = p.csec * 6,
            rout1    = clockRadius * 1.02,
            ch_indi  = p.secArrow or 7,
            cthick   = 4,
            length   = 1.2,
            chands   = 1,
            csize    = 0,
            icolour  = {p.secColour,1},
        }
    }

    for i in lpairs(ciData) do
        choose_indicator(ciData[i])
    end
    lcairo_restore(cr)

end

local function radial_pattern(p)
    local pat = lcairo_pattern_create_radial(
        p.xx, p.xy, p.decIn, p.xx, p.xy, p.decOut)
    lcairo_pattern_add_color_stop_rgba(pat, 0.0, to_rgba(p.col1))
    lcairo_pattern_add_color_stop_rgba(pat, 0.1, to_rgba(p.col2))
    lcairo_pattern_add_color_stop_rgba(pat, 1.0, to_rgba(p.col3))
    return pat
end

local function draw_clock_hands(p)

    local cr    = p.cr
    local xc    = p.xSun
    local yc    = p.ySun

    -- mag.sun_lha = 180

    local eclipticWidth   = 0.75			-- Width

    local hours_arc = toRadians(mag.sun_lha)

    local hourMoon  = (mag.moon_lha + 0) % 360

    local hourSun   = (mag.sun_lha + 180) % 360

    -- 20 is adjustment and 1.053 os the speed of ecliptic dial
    -- these need to be adjusted so that it matches moon and sun movements.
    -- https://www.moontracks.com/lunar_ingress.html for moon
    -- https://www.moontracks.com/sun_solar_ingress.html for Sun
    -- local testLha = ((mag.sun_lha - 30 ) * 1.051 ) % 360
    -- local testLha = ((mag.sun_lha - 27 ) * 1.04 ) % 360
    -- local testLha = ((mag.sun_lha - 27 ) * 1.02 ) % 360
    -- local testLha = ((mag.sun_lha - 27 ) * 1.03 ) % 360
    -- local testLha = ((mag.sun_lha - 27 ) * 1.0275 ) % 360
    -- lha = 15 * (lmstP(mjd0, ourlong, hour) - ra)
    local lmst = (mag.sun_lha / 15) + mag.sun_ra
    -- local testLha = (mag.sun_ra * 15) - 25
    local testLha = (lmst * 15) - 90

    local hourAries = toRadians( testLha )
    -- local hourAries = toRadians( (mag.sun_ra * 15) - 30)
    -- local hourAries = toRadians( (mag.sun_ra * 15) - 25)

    -- print( lmst , (lmst * 15) - 90)

    -- if p.csec % 60 == 0 then
    --     if time.min % 2 == 0 then
    --         t36 = testLha - hourSun
    --         print(2,testLha,hourSun, testLha - hourSun, t36 - t63)
    --     else
    --         t63 = testLha - hourSun
    --         print(1,testLha,hourSun, testLha - hourSun, t63 - t36)
    --     end
    -- end

    -- print(mag.sun_lha, (mag.sun_lha*2) % 360, mag.sun_lha - (mag.sun_lha*2) % 360)

    -- local ariesFactor = (time.yday + 5 + (p.chour / 24)) - 110
    -- local hourAriesA = toRadians((mag.sun_lha + ariesFactor + (360 - (mag.sun_ra * 15))) % 360)
    -- local hourAries = toRadians(ariesFactor)

    -- print(hourAries, ariesFactor, hourAriesA)
    -- print((360/365.25)*ariesFactor, ariesFactor, p.chour)

    -- ********** calculates Sun Declination ****************
    -- local least, fpoAries
    -- local dayRadian = toRadians(360/365.24)
    -- local sinTilt = sin(toRadians(-23.4393))
    -- local decHours = p.chour / 24
    -- local sFact = 2 * 0.0167

    -- we know the Spring Equinox falls around the 80th day
    -- so we use from yday 70 to 90
    -- Spring Equinox is the FPO Aries !!!
    -- for i = 70, 90 do
    --     local N = i + decHours - 1
    --     local sunDec = asin( sinTilt *
    --         cos(dayRadian * (N+10) + sFact *
    --         sin(dayRadian * (N-2))))
    --         -- Leave the values in Radians - be frugal ;-))
    --         -- sin(dayRadian * (N-2)))) *
    --         -- TODEGREES
    --     if sunDec < 0.01 and sunDec > -0.01 then
    --         local absVal = abs(sunDec)
    --         if not least or (absVal < least) then
    --             least = absVal
    --             local ri = time.yday - i
    --             fpoAries = os.date("%d", os.time() - (ri * 86400))
    --             -- print(i, sunDec)
    --         end
    --     end
    -- end
    -- print(fpoAries)
    ------- DRAW ECLIPTIC ---------------------------

    local Rd = 0.28467 * clockRadius
    local Re = 0.71533 * clockRadius

    local xe = xc + (Rd * sin(hourAries))
    local ye = yc - (Rd * cos(hourAries))

    -- Draw internal ring --

    lcairo_arc(cr,xe,ye,0.48 * (Re + eclipticWidth * Re ),0,TODEGREES)
    lcairo_set_source_rgba(cr,to_rgba({Blue,0.6}))
    lcairo_set_line_width(cr,35)
    lcairo_stroke(cr)

    -- Draw external rings --

    lcairo_arc(cr,xe,ye,Re,0,TODEGREES)
    lcairo_set_source_rgba(cr,to_rgba({Yellow,0.65}))
    lcairo_set_line_width(cr,2.5)
    lcairo_stroke(cr)

    lcairo_arc(cr,xe,ye,eclipticWidth * 1.25 * Re,0,TODEGREES)
    lcairo_set_source_rgba(cr,to_rgba({Yellow,0.65}))
    lcairo_set_line_width(cr,2.5)
    lcairo_stroke(cr)

    lcairo_arc(cr,xe,ye,eclipticWidth * Re * 0.9,0,TODEGREES)
    lcairo_set_source_rgba(cr,to_rgba({Yellow,0.65}))
    lcairo_set_line_width(cr,3)
    lcairo_stroke(cr)


    -- Zodiac Markings --

    local Lep = Re - Re * eclipticWidth

    local a, b, d, sd
    -- local ast = {'Ar','Ta','Ge','Ca','Le','Vi','Li','Sc','Sa','Ca','Aq','Pi'}
    local ast = {'a','b','c','d','e','f','g','h','i','j','k','l'}

    -- a = 0.28467 * clockRadius
    -- a = 0.2735 * clockRadius
    b = 2.51284

    local function toDegRad(x,f) return toRadians(x * f) end

    for i = 1,60 do
        local f = 30
        d  = toDegRad(i,f)
        sd = sin(d)
        a  = 0.285 * clockRadius
        Re = a * (sqrt((b * b) - (sd * sd)) + cos(d))
        local Lep1 = Lep + 9.5

        local factA    = hourAries + d
        local sinfactA = sin(factA)
        local cosfactA = cos(factA)
        local xa = xc + Re * sinfactA
        local ya = yc - Re * cosfactA
        local xb = xc + (Re - Lep1) * sinfactA
        local yb = yc - (Re - Lep1) * cosfactA

        lcairo_move_to(cr, xa, ya)
        lcairo_line_to(cr, xb, yb)


        lcairo_set_line_cap(cr,CAIRO_LINE_CAP_ROUND)
        lcairo_set_source_rgba(cr,to_rgba({Yellow,0.75}))
        lcairo_set_line_width(cr,1.5)
        lcairo_stroke(cr)

        -- the zodiac abreviations
        i  = i + 2.5
        d  = toDegRad(i,f)
        sd = sin(d) + 0.00
        a  = 0.275 * clockRadius
        Re = (a * (sqrt((b * b) - (sd * sd)) + cos(d))) - 3.9

        factA    = hourAries + d
        sinfactA = sin(factA)
        cosfactA = cos(factA)
        local xd = xc + Re * sinfactA
        local yd = yc - Re * cosfactA
        local xf = xc + (Re - Lep) * sinfactA
        local yf = yc - (Re - Lep) * cosfactA
        lcairo_set_source_rgba(cr,to_rgba({Yellow,0.85}))
        printText(cr,ast[lfloor(16-i)],(xd+xf)/2,(yd+yf)/2,1,21,false,false,"AstroDotBasic")
        lcairo_stroke(cr)

        f  = 6
        i  = i + 0.5
        d  = toDegRad(i,f)
        sd = sin(d)
        a  = 0.283 * clockRadius
        Re = a * (sqrt((b * b) - (sd * sd)) + cos(d))
        local Lep2 = 5

        factA    = hourAries + d
        sinfactA = sin(factA)
        cosfactA = cos(factA)
        local xg = xc + Re * sinfactA
        local yg = yc - Re * cosfactA
        local xh = xc + (Re - Lep2) * sinfactA
        local yh = yc - (Re - Lep2) * cosfactA
        lcairo_set_source_rgba(cr,to_rgba({Yellow,1}))
        lcairo_set_line_width(cr,2)
        lcairo_move_to(cr, xg, yg)
        lcairo_line_to(cr, xh, yh)
        lcairo_stroke(cr)
    end

    -- ---- hands ---------------------------------------------------


    -- Aries hand --
    local ariesHourFactor = hourAries + PI / 2
    local sinAriesHand    = sin(ariesHourFactor) * clockRadius
    local cosAriesHand    = cos(ariesHourFactor) * clockRadius
    local xa = xc + 0.78 * sinAriesHand
    local ya = yc - 0.78 * cosAriesHand
    local Xa = xc - 0.65 * sinAriesHand
    local Ya = yc + 0.65 * cosAriesHand
    lcairo_move_to(cr,xa,ya)
    lcairo_line_to(cr,Xa,Ya)

    lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND)
    lcairo_set_line_width(cr,3)
    lcairo_set_source_rgba(cr,to_rgba({DarkCyan,1}))
    lcairo_stroke(cr)

    lcairo_arc(cr,Xa,Ya,2.2,0,twoPI)
    lcairo_set_source_rgba(cr,to_rgba({Tomato,2}))
    lcairo_set_line_width(cr,4)
    lcairo_stroke(cr)

    lcairo_arc(cr,xa,ya,2.2,0,twoPI)
    lcairo_set_source_rgba(cr,to_rgba({Tomato,2}))
    lcairo_set_line_width(cr,4)
    lcairo_stroke(cr)

    ariesHourFactor = hourAries + PI / 270
    sinAriesHand    = sin(ariesHourFactor) * clockRadius
    cosAriesHand    = cos(ariesHourFactor) * clockRadius
    xa = xc + sinAriesHand
    ya = yc - cosAriesHand
    Xa = xc - 0.43 * sinAriesHand
    Ya = yc + 0.43 * cosAriesHand
    lcairo_move_to(cr,xa,ya)
    lcairo_line_to(cr,Xa,Ya)

    lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND)
    lcairo_set_line_width(cr,3)
    lcairo_set_source_rgba(cr,to_rgba({DarkCyan,1}))
    lcairo_stroke(cr)

    lcairo_arc(cr,Xa,Ya,2.2,0,twoPI)
    lcairo_set_source_rgba(cr,to_rgba({Tomato,2}))
    lcairo_set_line_width(cr,4)
    lcairo_stroke(cr)

    lcairo_arc(cr,xa,ya,2.2,0,twoPI)
    lcairo_set_source_rgba(cr,to_rgba({Tomato,2}))
    lcairo_set_line_width(cr,4)
    lcairo_stroke(cr)

    showTimeHands(p)

    --------- ASTRONOMICAL POSITIONS  ---------------------------------

    ----- SUN -----

    a = 0.277 * clockRadius
    b = 2.51284
    local d = - hourAries + hours_arc
    local sd = sin(d)
    local R = a * (sqrt(b * b - sd * sd) + cos(d))

    local Xs,Ys = get_points(0, hourSun, 0, R, 0, xc, yc)

    local bodySize = 0.245

    local function nightSun(alpha,f)

        local sunAlpha = tonum(mag.sun_alt)

        if sunAlpha < 0 then
            if f == 1 then
                sunAlpha = abs(alpha/(sunAlpha/12))
                -- sunAlpha = abs(alpha/(sunAlpha/25))
            else
                sunAlpha = abs(alpha/sunAlpha)
            end
        else
            sunAlpha = alpha
        end

        return sunAlpha

    end

    lcairo_save(cr)
    local decIn  = 9
    local decOut = 36
    local rpData = {
        xx     = Xs,
        xy     = Ys,
        decIn  = decIn ,
        decOut = decOut,
        col1   = { 0xffff55, nightSun(0.70) },
        -- col1   = { 0xffff55, nightSun(0.90,1) },
        col2   = { 0xff5555, 0.80 },
        col3   = { 0xffaa00, 0.05 },
        }
    local pat1 = radial_pattern(rpData)
    lcairo_arc(cr, Xs, Ys, decOut, TORADIANS, 0)
    lcairo_set_source(cr, pat1)
    lcairo_fill(cr)
    lcairo_pattern_destroy(pat1)
    lcairo_set_line_width(cr, 1)
    lcairo_set_source_rgba(cr, to_rgba({Yellow,1}))
    lcairo_arc(cr, Xs, Ys, decIn*1.1, TORADIANS, 0)
    lcairo_stroke(cr)
    lcairo_restore(cr)

    --------------------- Moon -----------------------

    local hour_MD = toRadians(hourMoon)

    -- a = 0.28467 * clockRadius
    a = 0.277 * clockRadius
    b = 2.51284
    local dm = hourAries - hour_MD
    local sdm = sin(dm)
    local Rm = a * (sqrt(b * b -sdm * sdm) + cos(dm))

    local Xm,Ym = get_points(0, hourMoon, 180, Rm , 0, xc, yc)
    lcairo_save(cr)

    local diffx = Xm - Xs
    local diffy = Ym - Ys
    local rotAng = TODEGREES * (atan2(-diffx, diffy))

    local moonImage, moonRotate, mcolour
    moonImage = tonum(mag.moonImage)

    if moonImage < 6 then
        moonImage = 6
    elseif moonImage > 23 and moonImage ~= 30 then
        moonImage = 23
    end
    if moonImage ~= 30 then
        mcolour = {LightGrey,0.45 / p.scale}
    end
    if moonImage == 15 or moonImage == 30 then
        moonRotate = 0
    elseif moonImage < 4 then
        moonImage = 3
        moonRotate = -135 + rotAng
    elseif moonImage > 26 then
        moonImage = 27
        moonRotate = 135 + rotAng
    elseif moonImage < 15 then
        if (moonImage % 2) == 0 then moonImage = moonImage + 1 end
        moonRotate = -135 + rotAng
    elseif moonImage > 15 then
        if (moonImage % 2) == 0 then moonImage = moonImage + 1 end
        moonRotate = 135 + rotAng
    end
    lcairo_restore(cr)

    lcairo_set_line_width(cr, 4)
    lcairo_arc(cr, Xm, Ym, 2.3/bodySize, TORADIANS, 0)
    -- lcairo_arc(cr, Xm, Ym, 2.95/bodySize, TORADIANS, 0)
    lcairo_set_source_rgba(cr, to_rgba({Cyan,1}) )
    lcairo_stroke_preserve(cr)
    lcairo_set_source_rgba(cr, to_rgba({Black,1}) )
    lcairo_fill(cr)

    -- if mcolour then mcolour = wxsplit(mcolour,':') end

    lcairo_save(cr)
    local piData = {
        cr     = cr,
        x      = Xm,
        y      = Ym,
        file   = tconcat({moonImage,'.png'}),
        scale  = bodySize*0.8,
        ip     = tconcat({ p.imagePath, '/daynight/' }),
        theta  = TORADIANS * moonRotate,
        rotate = true,
        -- dial_colour = mcolour ,
    }
    put_image(piData)
    lcairo_restore(cr)

end



---########################################################--
function w.astrolabe(p)

    local cr = p.cr

    clock24 = p.c or 24

    local location      = p.l     or '30:90'
          tzoneNum      = p.z     or tzoneNum
    local updateIn      = p.u     or nil
    local scale         = p.s     or 1
    local px            = p.lwx   or 0
    local py            = p.lwy   or 0
    local llabels       = p.a     or 2
    local fontSize      = p.f     or 12
    local labelFontSize = p.b     or 10
    local arrowColour   = p.g     or '0x55ff55:0xffff55:0xff55ff'
    local larrows       = p.r     or '7:7:7' -- hr:mn:sc
    local template      = p.t     or  nil

    if px == 0 then px = 250 else px = px + 250 end
    if py == 0 then py = 225 else py = py + 225 end

    if llabels == 0 then llabels = false end

    local _, count = lgsub(arrowColour, ":", "")
    _ = ''
    local arrowC = {true}
    if count == 2 then
        arrowC = wxsplit(arrowColour,':')
    end
    for i = 1, #arrowC do
        if tonum(arrowC[i]) == 0 then arrowC[i] = nil end
    end
    -- print(arrowC[3])

    _, count = lgsub(larrows, ":", "")
    _ = ''
    local arrow = {true}
    if count == 2 then
        arrow = wxsplit(larrows,':')
    end
    for i = 1, #arrow do
        if tonum(arrow[i]) == 0 then arrow[i] = nil end
    end

    updates  = tonum(lconky_parse("${updates}"))
    time     = ldate('*t')

    if tonum(updateIn) == 0 then updateIn = nil end

    local updateInterval
    if updateIn then
        updateInterval = updateIn
    else
        updateInterval = 60
    end

    local csec  = time.sec
    local cmin  = time.min  + (csec / 60)
    local chour = time.hour + (cmin / 60)


    if updates > 1 then

        local cwxMagellan

        local archType = ExecInfo({ cmd = 'uname -m' })
        if FileExists("./lib/magellan") then
            cwxMagellan = "./lib/magellan"
        elseif archType:match('x86_64') and FileExists("./lib/magellan_64") then
            cwxMagellan = "./lib/magellan_64"
        elseif FileExists("./lib/magellan_32") then
            cwxMagellan = "./lib/magellan_32"
        else
            error("magellan executable not found!")
        end

        -- start sunring --
        local wvar
        if tonum(conky_info.update_interval) < 10 then
            wvar = ( updates % updateInterval ) + 1
        else
            wvar = updateInterval
        end

        local posn = wxsplit(location,':')
        posn[1] = hrsToDec(posn[1])
        posn[2] = hrsToDec(posn[2])
        if tonum(posn[1]) < 0 then
            posn[1] = (posn[1] * -1) .. 'S'
        elseif tonum(posn[1]) > 0 then
            posn[1] = posn[1] .. 'N'
        end
        if tonum(posn[2]) < 0 then
            posn[2] = (posn[2] * -1) .. 'W'
        elseif tonum(posn[2]) > 0 then
            posn[2] = posn[2] .. 'E'
        end

        local mMonth = time.month
        if wvar == updateInterval or not magellan then
            magellan = ExecInfo({ cmd = tconcat({
                cwxMagellan,
                ' -y ',
                time.year,
                ' -m ',
                mMonth,
                ' -d ',
                time.day,
                ' -t ',
                tzoneNum,
                ' -o ',
                1,
                ' -a ',
                '%R',
                ' -b ',
                '"%b %d %R" ',
                ' ',
                posn[1],
                ' ',
                posn[2],
            }) })

            -- if debug then
            --     print(tconcat({'Debug update: ',time.hour,':',time.min,':',time.sec}))
            -- end
        end

        if magellan then
            local magval = split_newlines(magellan)
            -- =================== debug ===============================
            if debug and debug == 2 then
                ExecInfo({cmd = tconcat({'echo " " > /tmp/astroclock_debug'})})
            end
            for d = 1, #magval do
                local val = wxsplit(magval[d],'=')
                if val[1] and val[2] then
                    mag[val[1]] = lmatch(val[2],'"(.-)"')
                    -- =================== debug ===============================
                    if debug and debug == 2 and val[2] ~= '""' then
                        ExecInfo({cmd = tconcat({'echo ',val[1],' = ',val[2],' >> /tmp/astroclock_debug'})})
                    end
                end
            end
        end

        local params  = {
            cr            = cr,
            chour         = chour,
            cmin          = cmin,
            csec          = csec,
            xSun          = px,
            ySun          = py,
            radiusSun     = 196,
            radiusW       = 180,
            fontSize      = fontSize,
            labelFontSize = labelFontSize,
            hourColour    = arrowC[1],
            minColour     = arrowC[2],
            secColour     = arrowC[3],
            hourArrow     = (arrow[1] or 7),
            minArrow      = (arrow[2] or 7),
            secArrow      = (arrow[3] or 7),
            imagePath     = p.imgpath,
            scale         = scale,
            llabels       = llabels
        }
        lcairo_scale(cr, scale, scale)

        draw_sun_ring(params)

        lcairo_save(cr)
            drawLatLong(params)
        lcairo_restore(cr)

        drawMarksNumbers(params)

        if llabels then
            markerRST(params)
        end

        draw_clock_hands(params)

        lcairo_set_line_width (cr,2)
        lcairo_set_source_rgba (cr, to_rgba({DarkCyan,1})) --Arch Blue
        local radius = 12
        lcairo_arc (cr,px,py,radius,0,360)
        lcairo_stroke (cr)

        lcairo_set_source_rgba(cr, to_rgba({Tomato,1})) --Tomato
        radius = 5
        lcairo_arc (cr,px,py,radius,0,360)
        lcairo_fill (cr)

    end
    lcairo_scale(cr,1/scale,1/scale)

end

lcollectgarbage()

return w
