
--[[
    Paramvir Likhari - conkywx 2011 - 2020 - GPL3
    better image display with transparency
--]]

local w = { true }

local tonum           = tonumber
local lpairs          = pairs
local lcollectgarbage = collectgarbage
local lconky_parse    = conky_parse

local FC = require("luafunc")
local background  = FC.background
local FileExists  = FC.FileExists
local msgout      = FC.msgout
local ExecInfo    = FC.ExecInfo
local getDatafile = FC.getDatafile

local OS = require('os')
local lexecute   = OS.execute
local lremove    = OS.remove

local ST = require('string')
local lmatch     = ST.match
local lgsub      = ST.gsub

local TB = require('table')
local tconcat    = TB.concat

local ts = {}
local sNo

local cnTable = {}
local cn = 1
local function getCounter(infile)
    if not cnTable[infile] then
        cnTable[infile] = cn
        cn = cn + 1
    end
    for k, v in pairs(cnTable) do
        if k == infile then
            return v
        end
    end
end

----------------------------------------------------------------------
function w.display(p)

    sNo = getCounter(p.i)
    local tc = { [sNo] = sNo }
    if not ts[tc[sNo]] then ts[tc[sNo]] = 0 end
    ts[tc[sNo]] = ts[tc[sNo]] + 1

    local cr = p.cr
    if FileExists(p.i) then

        -- print(ts[tc[sNo]])

        local temppath = lmatch(p.i,'(.-)([^%/]-([^%.]+))$')
        local pngFile  = lgsub(p.i,'(.+%/.+)%..-$','%1.png',1)

        local loadingFile  = temppath .. 'loading'
        local showimagetmp = temppath .. 'showimage.tmp'
        -- local anyPng = temppath .. '*.png'

        if FileExists( loadingFile ) then
            lexecute( [[touch ]] .. showimagetmp )
        end

        if FileExists(showimagetmp) and
            not FileExists(loadingFile) or
            ts[tc[sNo]] == 3 then

            lremove(showimagetmp)
            lremove(pngFile)
        end

        if ExecInfo({ cmd = [[ type -p convert ]] }) ~= '' and
            not FileExists(pngFile) then
            lexecute(
                tconcat(
                    {
                        [[convert -resize 1000x700\! -gravity Center +repage -matte -channel A +level 0,]],
                        (p.a or 100),
                        [[% +channel ]],
                        p.i,
                        [[ ]],
                        pngFile,
                        [[ 2> /dev/null ]]
                    }
                )
            )
        end

        background({
            cr = cr,
            r  = 0,
            c  = '0x000000:0',
            i  = pngFile,
            e  = p.e,
            s  = p.s,
            p  = p.p or 0,
            b  = p.b,
        })
    else
        return nil
    end

end

function w.showimage(p)
    local cr = p.cr
    local dt, ds = {}, {}

    if not p.d then
        error('Error :( Please define the data file for showimage lua module.')
    end

    if p.d then
        local RD = getDatafile(p.d)
              dt = RD.dt
              ds = RD.ds
    end
    local dsSize = #ds

    local count = tonum(lconky_parse("${updates}")) + 1

    for i in lpairs(ds) do
        if ds[i].imageNo == count % dsSize then

            local fname = ds[i].path .. ds[i].name
            local pan   = ds[i].pan
            local errorImage = ds[i].path .. [[error.png]]
            if FileExists(fname) then
                lremove(errorImage)
            else
                if ExecInfo({ cmd = [[ type -p convert ]] }) ~= '' then
                    lexecute(
                        tconcat(
                            {
                                [[convert -background black -fill lightblue -font Ubuntu -size ]],
                                ds[i].errorImageSize,
                                [[ -pointsize 48  -gravity center label:]],
                                [[']],
                                ds[i].errorMsg,
                                [[']],
                                ' ',
                                errorImage,
                                [[ ]],
                                [[ 2> /dev/null ]],
                            }
                        )
                    )
                end
                fname = errorImage
                pan = '0:0'
            end

            w.display({
                cr = cr,
                r  = 0,
                c  = '0x000000:0',
                i  = fname,
                e  = ds[i].zoom,
                s  = ds[i].size,
                p  = ds[i].origin or 0,
                b  = pan,
                a  = ds[i].alpha,
                q  = ds[i].errorMsg,
                h  = ds[i].path,
            })
        end
    end
end


lcollectgarbage()

return w
