package wxparser;

=comments
    Parses Weather current and forecast
    This version (c) 2018 Paramvir Likhari
=cut

$wxparser::VERSION = 200228;

require Exporter;
@wxparser::ISA = qw(Exporter);
@wxparser::EXPORT = qw(wxParserCurrent); # function to export

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

my ( %fctinfo, $temppath, $currdatam, $noFileMsg, $wxMainTitle, $wxFctTitle, $darkSkyLoc, $userLoc, $monAbb, $ltime, $year, $icao_id, $alreadyHaveForecast, $fctString, %wundFct, $waqidata, $wundoption, $fctBackUrl, %fctBackup, $latitude, $longitude, $elevation, %fctM, %fctB ) ;

my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);

my %msky = (
    BC     => "Patches",          BL  => "Blowing",       BR   => "Mist",
    DR     => "Low Drifting",     DS  => "Duststorm",     DU   => "Dust",
    DZ     => "Drizzle",          FC  => "Funnel Cloud",  FG   => "Fog",
    FU     => "Smoke",            FZ  => "Freezing",      GR   => "Hail",
    GS     => "Snow Pellets",     HZ  => "Haze",          IC   => "Ice Crystals",
    MI     => "Shallow",          NA  => "NA",            PL   => "Ice Pellets",
    PO     => "Dust/Sand Whirls", PR  => "Partial",       PY   => "Spray",
    RA     => "Rain",             SA  => "Sand",          SG   => "Snow Grains",
    SH     => "Showers",          SN  => "Snow",          SQ   => "Squalls",
    SS     => "Sandstorm",        TS  => "Thunderstorm",  UP   => "Unknown Precipitation",
    VA     => "Volcanic Ash",     VC  => "Nearby",        OVC  => "Overcast",
    BKN    => "Mostly Cloudy",    SCT => "Partly Cloudy", FEW  => "Clear",
    CLR    => "Clear",            SKC => "Clear",         CAVU => "Clear",
    CAVOK  => "Clear",
);


sub darkskyFctParser {

    my ( $fileToRead ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my %fct ;
    my $dsFct = ${main::get_line_num( $fileToRead, '<div class="dayDetails">','<div id="timeMachine"', 'str')};

    $dsFct =~ tr/\x0a\x20/ /s ;
    my $cntn = 0;
    while ($dsFct =~ m{<div class="summary">(.*?)</div>}g) {
        push(@$font,main::do_get_DI(2, $1,$cntn));
    }
    while ($dsFct =~ m{<span class="num swip">(.*?)</span>}g) {
        my $cor1 = ( $1 ) ? $1 * 100 : ' 0' ;
        push(@$cor, $cor1 );
        push(@$hum, "NA");
        push(@$corhum, " $cor1|NA");
    }
    $cntn = 0;
    while ($dsFct =~ m{<span class="highTemp.*?"temp">(.*?)˚</span>.*?="lowTemp.*?="temp">(.*?)˚</span>}g) {
        push(@$temp, main::ch_fcunits("$2/$1"));
    }
    $cntn = 0;
    while ($dsFct =~ m{href="\/details\/.*?\/(.*?)\/us24\/en"}g) {
        push(@$day, main::get_short_day(main::time2str("%a %d",$1)));
    }


    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

   return %fct
}

sub yahooFctParser {

    my ( $fileToRead ) = @_ ;

    my $yFct = ${main::get_line_num( $fileToRead,'"daily":','"precipitations":','str')};
    $yFct =~ s/^/{/;
    $yFct =~ s/,$//;

    # msgout($yFct);

    my $yhFct = main::jsonParser( $yFct );

    my %wkday = (
        '1' => 'Mon',
        '2' => 'Tue',
        '3' => 'Wed',
        '4' => 'Thu',
        '5' => 'Fri',
        '6' => 'Sat',
        '0' => 'Sun',
    );

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];

    my %fct ;

    my $cntn = 1;
    for my $k (0..$#{$$yhFct{daily}}) {

        my $tMax = $$yhFct{daily}[$k]{temperature}{high} ||= ' NA';
        my $tMin = $$yhFct{daily}[$k]{temperature}{low}  ||= 'NA ';
        push(@$temp,main::ch_fcunits("$tMax/$tMin"));

        my $cor1 = (" $$yhFct{daily}[$k]{precipitationProbability}") ? " $$yhFct{daily}[$k]{precipitationProbability}" : 'NA';
        push(@$cor, " $cor1");

        my $rhum = $$yhFct{daily}[$k]{humidity} ||= 'NA ';
        push(@$hum, $rhum);

        my $dy = $$yhFct{daily}[$k]{observationTime}{day} ||= 'NA';
        my $wd = ( $$yhFct{daily}[$k]{observationTime}{weekday} ) ? $wkday{$$yhFct{daily}[$k]{observationTime}{weekday}} : 'Sun';
        push(@$day,main::get_short_day("$wd $dy"));

        my $cond1 = $$yhFct{daily}[$k]{conditionDescription} ||= ' NA';
        push(@$font,main::do_get_DI(2, $cond1 ,$cntn));

        push(@$corhum,"$cor1|$rhum");

        $cntn++;

    }

    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    return %fct
}

sub weatherFctParser {

    my ( $fileToRead, $opt ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my (%fct);

    $$fileToRead =~ tr/\\//d;

    my $xml1 = ${main::get_line_num( $fileToRead,'"getSunV3DailyForecastUrlConfig"','"getSunV3TileServerSeriesDynamicMapsUrlConfig"','str')};
    # $xml1 =~ tr/\\//d;

    $xml1 =~ s/.*?"data":/{"data":/;
    $xml1 =~ s/,"status":200,.*$/}/;
    # msgout($xml1);

    my $wxfct = main::jsonParser( $xml1 );

    for my $k (0..$#{$$wxfct{data}{dayOfWeek}}) {
        if ($$wxfct{data}{sunriseTimeLocal}[$k] =~ m{.*?-.*?-(.*?)T.*}) {
            push(@$day,main::get_short_day("$$wxfct{data}{dayOfWeek}[$k] $1"));
        }

        my $tMax = ($$wxfct{data}{temperatureMax}[$k]) ? $$wxfct{data}{temperatureMax}[$k] : 'NA';
        my $tMin = ($$wxfct{data}{temperatureMin}[$k]) ? $$wxfct{data}{temperatureMin}[$k] : 'NA';

        push(@$temp,main::ch_fcunits("$tMax/$tMin"));
    }

    my $cntn = 0;
    for my $k (0..$#{$$wxfct{data}{daypart}[0]{precipChance}}) {
        if ($k % 2 == 0) {
            my $cor1 = ($$wxfct{data}{daypart}[0]{precipChance}[$k]) ? $$wxfct{data}{daypart}[0]{precipChance}[$k] : '0';
            push(@$cor, " $cor1");

            my $rhum = ($$wxfct{data}{daypart}[0]{relativeHumidity}[$k]) ? $$wxfct{data}{daypart}[0]{relativeHumidity}[$k] : 'NA';
            push(@$hum, $rhum);

            my $cond1 = ($$wxfct{data}{daypart}[0]{wxPhraseLong}[$k]) ? $$wxfct{data}{daypart}[0]{wxPhraseLong}[$k] : 'Sunny' ;
            push(@$font,main::do_get_DI(2, $cond1 ,$cntn));
            $cntn++;

            push(@$corhum,"$cor1|$rhum");
        }
    }
    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    # foreach my $k (keys %fct) {
    #     my $count = 0;
    #     foreach (@{$fct{$k}}) {
    #         msgout($count++, $k, "$_");
    #     }
    # }

    return %fct
}

sub openwxmapFctParser {

    my ( $dataToRead ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my (%fct);

    # msgout($$dataToRead{daily}[0]{dt});
    # msgout($#{$$dataToRead{daily}});

    my $cntn = 0;
    for my $k (0..$#{$$dataToRead{daily}}) {
        # if ($k % 2 == 0) {
            push(@$day, main::get_short_day(main::time2str("%A %d",$$dataToRead{daily}[$k]{dt})));

            my $tMin = ($$dataToRead{daily}[$k]{temp}{min}) ? sprintf("%1.0f",$$dataToRead{daily}[$k]{temp}{min}) : '0';
            my $tMax = ($$dataToRead{daily}[$k]{temp}{max}) ? sprintf("%1.0f", $$dataToRead{daily}[$k]{temp}{max}) : '0';
            push(@$temp,main::ch_fcunits("$tMax/$tMin"));

            my $cor1 = ($$dataToRead{daily}[$k]{pop}) ? $$dataToRead{daily}[$k]{pop} * 100 : '0';
            push(@$cor, " $cor1");

            my $rhum = ($$dataToRead{daily}[$k]{humidity}) ? $$dataToRead{daily}[$k]{humidity} : 'NA';
            push(@$hum, $rhum);

            my $cond1 = ($$dataToRead{daily}[$k]{weather}[0]{main}) ? $$dataToRead{daily}[$k]{weather}[0]{main} : 'Sunny' ;
            push(@$font,main::do_get_DI(2, $cond1 ,$cntn));
            $cntn++;

            # msgout("$cor1|$rhum");
            push(@$corhum,"$cor1|$rhum");
        # }
    }
    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    # foreach my $k (keys %fct) {
    #     my $count = 0;
    #     foreach (@{$fct{$k}}) {
    #         msgout($count++, $k, "$_");
    #     }
    # }

    return %fct
}


sub wundergroundFctParser {

    my ( $fileToRead, $opt ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my %fct ;

    my $wxfct = main::jsonParser( $$fileToRead );

    my $cntn = 0;
    for my $k (0..$#{$$wxfct{forecasts}}) {
        if ($k+1 <= $#{$$wxfct{forecasts}})  {
            if ((($k == 0) and ($$wxfct{forecasts}[$k]{day_ind} eq 'N')) or ($$wxfct{forecasts}[$k]{day_ind} eq 'D')) {

                push(@$font, main::do_get_DI(2, ($$wxfct{forecasts}[$k]{phrase_32char} ||= 'NA'), $cntn));

                my $tHi = $$wxfct{forecasts}[$k]{temp} ||= ' --';

                my $tLo = $$wxfct{forecasts}[$k+1]{temp} ||= '-- ';

                push(@$temp, main::ch_fcunits("$tHi/$tLo"));

                my $cor1 = $$wxfct{forecasts}[$k]{pop} ||= ' 0' ;

                push(@$cor, $cor1);

                my ($date1) = $$wxfct{forecasts}[$k]{fcst_valid_local} =~ m{.*?-.*?-(.*?)T.*$};

                my $day1 = $$wxfct{forecasts}[$k]{long_daypart_name} ||= ' NA';

                push(@$day, main::get_short_day("$day1, $date1"));

                my $hum1 =  $$wxfct{forecasts}[$k]{rh} ||= 'NA ' ;
                push(@$hum, $hum1);

                push(@$corhum, "$cor1|$hum1");

                $cntn++;

            }
        }
    }

    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    # foreach my $k (keys %fct) {
    #     my $count = 0;
    #     foreach (@{$fct{$k}}) {
    #         msgout( $k, "$_");
    #     }
    # }

    return %fct
}


sub timeDateFctParser {

    my ( $fileToRead ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my (%fct);

    my $tndstr = ${main::get_line_num( $fileToRead,'var data={"','};','str')};

    $tndstr =~ s/var data=\{/\{"data":\{/;
    $tndstr =~ s/$/}}/;

    # msgout($tndstr);

    my $wxfct = main::jsonParser( $tndstr );

    my $cntn = 0;
    for my $k (0..$#{$$wxfct{data}{detail}}) {

        my $day1 = ($$wxfct{data}{detail}[$k]{ts}) ? $$wxfct{data}{detail}[$k]{ts} : 'NA';
        $day1 =~ s/,//;
        push(@$day, $day1 );

        push(@$font,main::do_get_DI(2, ($$wxfct{data}{detail}[$k]{desc} ||= 'NA') ,$cntn));

        my $th = $$wxfct{data}{detail}[$k]{temp}    ||= 'NA';
        my $tl = $$wxfct{data}{detail}[$k]{templow} ||= 'NA';
        push(@$temp,main::ch_fcunits("$th/$tl"));

        my $cor1 = (" $$wxfct{data}{detail}[$k]{pc}") ? " $$wxfct{data}{detail}[$k]{pc}" : 'NA';
        push(@$cor, " $cor1");

        my $rhum = ($$wxfct{data}{detail}[$k]{hum}) ? " $$wxfct{data}{detail}[$k]{hum}" : 'NA';
        push(@$hum, $rhum);
        # msgout($day1,$rhum);

        push(@$corhum, "$cor1|$rhum");
    }

    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    # foreach my $k (keys %fct) {
    #     my $count = 0;
    #     foreach (@{$fct{$k}}) {
    #         msgout( $k, "$_");
    #     }
    # }

    return %fct
}


sub accuweatherFctParser {

    my ( $fileToRead ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my %fct ;

    my $fctString = ${main::get_line_num( $fileToRead, '"forecast-list-card forecast-card  today"', '</html>', 'str', 1 )};

    $fctString =~ tr/\x0a\x20/\x20/s;

    # msgout($fctString);

    my $cntn = 1;
    while ($fctString =~ m{<span class="phrase">\s*(.*?)\s*</span>}g) {
        my $val = ($1) ? $1 : 'NA' ;
        push(@$font,main::do_get_DI(2, $val ,$cntn));
        $cntn++;
    }

    while ($fctString =~ m{<div class="info precip">\s*<p>\s*Precip\s*</p>\s*<p>(.*?)%</p>}g) {
        my $cor1 = ( $1 ) ? $1 : ' 0' ;
        push(@$cor, " $cor1");
        push(@$hum, "NA");
        push(@$corhum, " $cor1|NA");
    }

    while ($fctString =~ m{<div class="temps">\s*<span class="high"> (.*?)&#xB0; </span>\s*<span class="low"> / (.*?)&#xB0; </span>}g) {
        my ($ls,$rs) = ($1,$2);
        if (index($rs,'tonight') != -1 ) {
            $ls =~ tr/0-9.-//cd ;
            ($rs) = ($ls ne '') ? $ls : '--' ;
            $ls = '--';
        }
        else {
            $ls =~ tr/0-9.-//cd ;
            ($ls) = ($ls ne '') ? $ls : '--' ;
            $rs =~ tr/0-9.-//cd ;
            ($rs) = ($rs ne '') ? $rs : '--' ;
        }
        push(@$temp, main::ch_fcunits("$ls/$rs"));
    }
    while ($fctString =~ m{	<div class="date">\s*<p class="dow">\s*(.*?)\s*</p>\s*<p class="sub">\s*.*?/(.*?)\s*</p>\s*</div>}g) {
        push(@$day, main::get_short_day("$1 $2"));
    }


    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    # foreach my $k (keys %fct) {
    #     my $count = 0;
    #     foreach (@{$fct{$k}}) {
    #         msgout( $k, "$_");
    #     }
    # }

    return %fct
}


sub msnFctParser {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

=comments
    the hash %wxta uses 4 parts - delimiter is ';;'
    1st = start context
    2nd = end context
    3rd = type of data expected
    4th = optional - use 1 to use rindex in place of index to search
=cut

    my ( $fileToRead ) = @_ ;

    my ($day, $temp, $cor, $hum, $font, $corhum) = [];
    my (%fct, $fctl) ;


    my $fctdb = ${main::get_line_num( $fileToRead, '<ul class="forecast-list">',"normalsection", 'str')};

    for my $i (split('\n', $fctdb)) {
        # my $k = $$fctdb[$i];
        my $k = $i;
        if (index($k,'aria-label') != -1 )     {
            $fctl .= $k
        }
        if (index($k,'"image skyimg"') != -1 ) {
            $fctl .= $k
            }
        if (index($k,'<div title="Precipitation" class="precipicn') != -1) {
            $fctl .=$k.";;;"
        }
    }

    my $cntn = 0;
    my ($delim, %wxta);
    my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);
    $delim = ';;;';
    %wxta = (
        day   => [ 'for ',            ' is',        'd'  ],
        cond  => [ '"image skyimg" ', ' data-icon', 'c'  ],
        temp  => [ 'High of ',        '<div title', 't' ],
        pop   => [ 'Precipitation',   '</span>',    'p'  ],
    );
    # $alreadyHaveForecast =~ tr/\x0a\x20/ /s ;

    # msgout($fctl);

    for ( split /$delim/, $fctl ) {
        # msgout('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'.$_);
        next if /Early AM/;
        for my $ky (keys %wxta) {
            # my $val;
            my $st     = $wxta{$ky}[0];
            my $nd     = $wxta{$ky}[1];
            my $type   = $wxta{$ky}[2];
            my $offset = $wxta{$ky}[3];
            $st  = index($_,$st) + length($st);;
            if ($offset) {
                $nd  = rindex($_,$nd) - $st
                }
            else {
                $nd  = index($_,$nd, $st) - $st
                }
            my $fnd = substr($_, $st, $nd );
            for ( $type ) {
                # msgout($ky,$st, $nd,$fnd);
                if    ( /d/ )  { # dates day
                    push(@$day, main::get_short_day($fnd));
                }
                elsif ( /c/ )  { # alphanumeri with space cond
                    if ($fnd =~ m{title="(.*?)"}) {
                        push(@$font,main::do_get_DI(2, $1 ,$cntn));
                    }
                }
                elsif ( /t/ ) { # numbers only - high low temps
                    if ($fnd =~ m{(--?|(-?([\.\d]+)?)).*(?:low of )(--?|(-?([\.\d]+)?))}) {
                        push(@$temp, main::ch_fcunits("$1/$4"));
                    }
                }
                elsif ( /p/ )  { # numbers only - pop
                    if ($fnd =~ m{(\d+)%}) {
                        my $cor1 = " $1" ;
                        push(@$cor, " $cor1");
                        push(@$hum, "NA");
                        push(@$corhum, " $cor1|NA");
                    }
                }
            }
        }
    }
    $fct{day}    = $day;
    $fct{temp}   = $temp;
    $fct{cor}    = $cor;
    $fct{hum}    = $hum;
    $fct{font}   = $font;
    $fct{corhum} = $corhum;

    # foreach my $k (keys %fct) {
    #     my $count = 0;
    #     foreach (@{$fct{$k}}) {
    #         msgout( $k, "$_");
    #     }
    # }

    return %fct
}


sub getCurrent {

    my ($instr, $wxMainTitle) = @_;
    # msgout($instr);

    my (%wxta,%wxtb, %wxLocal);
    if ( $wxMainTitle eq 'msn' ) {
        %wxta = (
            city             => [ '"placeName" title=', '>',    'al' ],
            latitude         => [ ';lat=',          '&amp;l',   'n'  ],
            longitude        => [ ';long=',         '&amp;c',   'n'  ],
            temperature      => [ 'temperature is', 'degrees',  'n'  ],
            condition        => [ 'condition is ',  ' and the', 'a'  ],
            wind_dir         => [ 'class="winddir', '</li>',    'w'  ],
            # wind_speed       => [ 'winddir',        '</li>',    'n'],
            visibility       => [ 'Visibility',     '</li>',    'n'  ],
            pressure         => [ 'Barometer',      '</li>',    'n'  ],
            humidity         => [ 'Humidity',       '%',        'n'  ],
            dewpoint         => [ 'Dew Point',      '</li>',    'n'  ],
            name             => [ '"placeName" title=', '>',    'al' ],
            nameid           => 'MSNWX',
            # elevation        => 'NA',
            # condition        => 'NA',
            # icon             => 'NA',
            # precip_today     => 'NA',
            # snow             => 'NA',
            # wind_dir_degrees => 'NA',
            # wind_gust_speed  => 'NA',
            # cloud1h          => 'NA',
            # cloud1t          => 'NA',
            # updated          => 'NA',
            # solarRad         => 'NA',
        );
    }
    if ( $wxMainTitle eq 'accu' ) {
        %wxta = (
            city             => [ '<title>',      ',',     'a'  ],
            latitude         => [ '<span class="latitude">','</span>', 'n' ],
            longitude        => [ '<span class="longitude">','</span>', 'n' ],
            temperature      => [ '<div classtemperatures><p classvalue>','xB0','n'],
            condition        => [ '<div classphrase>',  '</div>',   'a'  ],
            wind_dir_degrees => [ 'transform: rotate', 'deg',  'n'  ],
            wind_speed       => [ 'Wind: ',    '</p', 'n'  ],
            wind_gust_speed  => ['<p>Gusts:',            '</p>', 'n'  ],
            visibility       => [ 'Visibility:',              '</p>', 'n'  ],
            pressure         => [ 'Pressure: ',               '</p>', 'n'  ],
            humidity         => [ 'Humidity',                 '</p>', 'n'  ],
            dewpoint         => [ 'Dew Point',                '</p>', 'n'  ],
            cloud1t          => [ 'Cloud Cover: ',             '</p>', 'n'  ],
            cloud1h          => [ 'Ceiling:',                 '</p>', 'n'  ],
            name             => [ '<title>',      ',',     'a'  ],
            nameid           => 'AccuWX',
            # elevation        => 'NA',
            icon             => [ '<div classphrase>',  '</div>',   'a'  ],
            # precip_today     => 'NA',
            # snow             => 'NA',
            # wind_dir         => 'NA',
            # updated          => 'NA',
            # solarRad         => 'NA',
        );
    }
    if ( $wxMainTitle eq 'timeDate' ) {
        %wxta = (
            updated          => ['<span id=wtct>',       '</span></p>', 'n1' ],
            city             => ['alt="Location of ',        '" style=', 'al'],
            name             => ['id=wt-loc title="',        '">',       'al'],
            condition        => ['class=mtt title="',        '." src="', 'al'],
            visibility       => ['Visibility: </span> ',     '&nbsp;',   'n' ],
            humidity         => ['Humidity: </span> ',       '%</p>',    'n' ],
            dewpoint         => ['Dew Point: </span> ',      '&nbsp;',   'n' ],
            temperature      => ['<div class=h1>Now ',       '&nbsp;',   'n' ],
            pressure         => ['Pressure: </span> ',       '</p>',     'n' ],
            wind_dir_degrees => ['title="Wind blowing from ','° ',       'n' ],
            wind_speed       => ['<br>Wind: ',               ' <span',   'n' ],
            nameid           => 'timeanddate.com',
            latitude         => ($latitude)  ? $latitude  : 'NA',
            longitude        => ($longitude) ? $longitude : 'NA',
            elevation        => ($elevation) ? $elevation : 'NA',
            icon             => ['class=mtt title="',        '." src="', 'al'],
            # precip_today     => 'NA',
            # snow             => 'NA',
            # wind_dir         => 'NA',
            # wind_gust_speed  => 'NA',
            # cloud1h          => 'NA',
            # cloud1t          => 'NA',
            # solarRad         => 'NA',
        );
    }

    my (%idxl, @idxarr, %idxl2);
    $instr =~ tr/\x0a\x20/ /s ;
    # $instr =~ s/&nbsp;/ /g;
    # msgout('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>',$instr);
    for my $ky ( keys %wxta) {
        if (ref($wxta{$ky}) eq 'ARRAY') {
            my $st   = $wxta{$ky}[0];  # string where it starts
            my $nd   = $wxta{$ky}[1];  # string where it ends
            my $type = $wxta{$ky}[2];  # what to do with it
            my $st1  = index($instr, $st) + length($st); # get posn
            my $nd1  = index($instr, $nd, $st1);  # get number of char to pull
            my $fnd  = substr($instr, $st1, $nd1-$st1 );

            # msgout($ky,$st1, $nd1,'>>>'.$fnd.'<<<' );

            if    ($type eq 'n' ) {
                $wxLocal{$ky} = ($fnd =~ m{(-?(\d+(\.\d+)?))}) ? $1 : 'NA'
                }
            elsif ($type eq 'd') {
                $wxLocal{$ky} = ($fnd =~ m{,\s*(-?(\d+(\.\d+)?))}) ? $1 : 'NA'
                }
            elsif ($type eq 'n1') {
                $wxLocal{$ky} = ($fnd =~ m{(.*+)}) ? $1 : 'NA' ;
                # msgout($fnd,'jjj',$wxLocal{updated});
                }
            elsif ($type eq 'a' ) {
                $wxLocal{$ky} = ($fnd =~ m{(\w+((([,|\-|\s]+\w+))+)?)}) ? $1 : 'NA' ;
                if ($wxLocal{wind_dir}) {
                    ($wxLocal{wind_dir_degrees},$wxLocal{wind_dir}) = main::find_wind_dir($wxLocal{Localwind_dir})
                }
            }
            elsif ($type eq 'al') {
                if ($wxLocal{wind_dir_degrees} and not exists $wxLocal{wind_dir} ) {
                    $wxLocal{wind_dir} = main::find_wind_dir($wxLocal{wind_dir_degrees});
                }
                $wxLocal{$ky} = $fnd ;
            }
            elsif ($type eq 'w' ) {
                ($wxLocal{wind_dir},$wxLocal{wind_speed}) = ($fnd =~ m{winddir.*?>(.*?)</div>(.*?)mph}) ? ($1,$2) : ("NA","NA")   ;
                if ($wxLocal{wind_dir}) {
                    ($wxLocal{wind_dir_degrees},$wxLocal{wind_dir}) = main::find_wind_dir($wxLocal{Localwind_dir});
                }
            }
        }
        else {
            $wxLocal{$ky} = $wxta{$ky}
        }
        # msgout('ppp',$wxLocal{updated});

    }
    if ( $wxMainTitle eq 'accu' ) {
        # msgout($wxLocal{cloud1t});
        $wxLocal{cloud1t} = ($wxLocal{cloud1t}) ? $wxLocal{cloud1t}.'%' : '0%'
    }

    return \%wxLocal
}

sub wxParserForecast {

    my ($fctstr, $histCallStr);
    my $fct_data = "$temppath/fct_data.html" ;

    my (%fct);

    if ( -e $fct_data ) {
        {
        open (FILE, '<:crlf', "$fct_data") or die "$noFileMsg forecast data file $!";
            local $/ = undef;
            $fctstr = <FILE> ;
        close (FILE);
        }

        # msgout($fctstr);

        if ( $wxFctTitle eq "darkSky" ) {
            %fct = darkskyFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "yahoo" ) {
            %fct = yahooFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "timeDate" ) {
            %fct = timeDateFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "wund" ) {
            %fct = wundergroundFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "msn" ) {
            %fct = msnFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "accu" ) {
            %fct = accuweatherFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "wx" ) {
            %fct = weatherFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "openwxmap" ) {
            %fct = openwxmapFctParser( \$fctstr )
        }
    }
    return %fct;
}

sub msnWxParser {

    my ( $fileToRead ) = @_ ;

    my ($wxLocal);

    my $msnwarn = ${main::get_line_num( $fileToRead, 'class="alerts','div','str')};
    if ($msnwarn) {
        my $c;
        foreach(split('</div>',$msnwarn)) {
            if ( m{<h2>(.*?)<span id="alertTime">(.*?)</span>} ){
                $c++; $$wxLocal{alertdata} .= "Weather alert $c: ".$1.$2.' | ';
            }
        }
    }
    my ( $msn1 ) = ${main::get_line_num( $fileToRead, 'weathermain','"DailyForecast"', 'str')};

    $wxLocal = getCurrent($msn1, $wxMainTitle);

    return \%$wxLocal
}

sub darkskyWxParser {

    my ( $fileToRead, $opt ) = @_ ;

    my $wxMain = ${main::get_line_num( $fileToRead, 'currently =','map_info', 'str')};

    $wxMain =~ s/currently = /{"data":/;
    $wxMain =~ s/,\s*$/}/;

    # msgout($wxMain);
    my $wxM1 = main::jsonParser( $wxMain );

    my %wxLocal = (
        updated          => $$wxM1{data}{time}        ||= 'NA',
        cloud1t          => $$wxM1{data}{cloudCover}  ||= 'NA',
        temperature      => $$wxM1{data}{temperature} ||= 'NA',
        dewpoint         => $$wxM1{data}{dewPoint}    ||= 'NA',
        condition        => $$wxM1{data}{summary}     ||= 'NA',
        pressure         => $$wxM1{data}{pressure}    ||= 'NA',
        wind_speed       => $$wxM1{data}{windSpeed}   ||= 'NA',
        wind_gust_speed  => $$wxM1{data}{windGust}    ||= 'NA',
        wind_dir_degrees => $$wxM1{data}{windBearing} ||= 'NA',
        icon             => $$wxM1{data}{icon}        ||= 'NA',
        humidity         => $$wxM1{data}{humidity}    ||= 'NA',
        visibility       => $$wxM1{data}{visibility}  ||= 'NA',
        solarRad         => $$wxM1{data}{solar}{ghi}  ||= 'NA',
        city             => 'NA',
        name             => 'NA',
        nameid           => 'darkSky',
        latitude         => ($latitude)  ? $latitude  : 'NA',
        longitude        => ($longitude) ? $longitude : 'NA',
        elevation        => ($elevation) ? $elevation : 'NA',
        precip_today     => 'NA',
        snow             => 'NA',
        wind_dir         => 'NA',
        cloud1h          => 'NA',
    );

    # msgout($wxLocal{wind_gust_speed});

    if ( $wxLocal{wind_dir_degrees} ne 'NA' ) {
        $wxLocal{wind_dir} = main::find_wind_dir($wxLocal{wind_dir_degrees})
    }

    if ( $wxLocal{pressure} ne 'NA' ) {
        $wxLocal{pressure} = main::hPa_in($wxLocal{pressure})
    }

    if ( $wxLocal{humidity} and index($wxLocal{humidity}, '0.') != -1 ) {
        $wxLocal{humidity} = $wxLocal{humidity} * 100
    }

    $wxLocal{solarRad} = (($wxLocal{solarRad}) and ($wxLocal{solarRad} ne 'NA')) ? main::round($wxLocal{solarRad}) : 'NA' ;

    if ($opt) {
        $wxLocal{city} = $wxLocal{name} = $opt;
    }
    else {
        $wxLocal{city} = (not $wxLocal{city}) ? $userLoc : $wxLocal{city};
        $wxLocal{name} = $wxLocal{city};
    }

    $wxLocal{cloud1t} = (($wxLocal{cloud1t}) and (main::check_num($wxLocal{cloud1t}) ne 'NA') and (index($wxLocal{cloud1t}, '%') == -1)) ? ($wxLocal{cloud1t} * 100) . '%' : ($wxLocal{cloud1t} ) ? $wxLocal{cloud1t} : 'NA';

    return \%wxLocal

}

sub wundergroundWxParser {

    my ( $fileToRead ) = @_ ;

    my ($wundwarn, %wundh, %wxLocal);

    if (index($fileToRead,'class="alert-items">') != -1) { $wundwarn=1 }
    # @wundhash{@wundarr} = @base_wx_list;
    if ($wundwarn) {
        my $c;
        for my $k (split('<span>', ${main::get_line_num( $fileToRead,'<div class="alert-items">','</div>','str')})) {
            $k =~ s/<+[^>,]*>//g;
            next if $k =~ /^\s*$/;
            $c++;
            $wxLocal{alertdata} .= "Weather alert $c: $k | ";
        }
    }

    my $wcData = ${main::get_line_num($fileToRead,'{&q;location&q;:',',&q;adminDistrictCode&q;','str')};
    $wcData .= ',';

    $wcData .= ${main::get_line_num($fileToRead,'&q;observations&q;',',&q;url&q;:&q;https://api.weather.com/v2/pws/observations/current?','str')};
    $wcData =~ s/,&q;observations&q;/},&q;pws&q;/;

    $wcData .= ${main::get_line_num($fileToRead,'{&q;cloudCeiling&q;',',&q;url&q;:&q;https://api.weather.com/v3/wx/observations/current?apiKey','str')};
    $wcData =~ s/&q;/"/g;
    $wcData =~ s#\{"cloudCeiling"#"airport":\{"cloudCeiling"#;
    $wcData =~ s/,"expiresAt":".*?Z",?//g;
    $wcData =~ s/\[\{/\{/;
    $wcData =~ s/}]}/},/;
    $wcData =~ s/$/}/;

    # msgout($wcData);

    # die;
    my $wcData2 = main::jsonParser( $wcData );
    # msgout($$wcData2{location}{airport}{temperature});

    if ($wundoption eq 'pws') {
        %wxLocal = (
            city             => $$wcData2{location}{city}            ||= 'NA',
            name             => $$wcData2{location}{adminDistrict}   ||= 'NA',
            nameid           => $$wcData2{pws}{stationID}            ||= 'NA',
            latitude         => $$wcData2{pws}{lat}                  ||= 'NA',
            longitude        => $$wcData2{pws}{lon}                  ||= 'NA',
            updated          => $$wcData2{pws}{epoch}                ||= 'NA',
            humidity         => $$wcData2{pws}{humidity}             ||= 'NA',
            wind_dir_degrees => $$wcData2{pws}{winddir}              ||= 'NA',
            solarRad         => $$wcData2{pws}{solarRadiation}       ||= 'NA',
            elevation        => $$wcData2{pws}{imperial}{elev}       ||= 'NA',
            temperature      => $$wcData2{pws}{imperial}{temp}       ||= 'NA',
            dewpoint         => $$wcData2{pws}{imperial}{dewpt}      ||= 'NA',
            pressure         => $$wcData2{pws}{imperial}{pressure}   ||= 'NA',
            wind_speed       => $$wcData2{pws}{imperial}{windSpeed}  ||= 'NA',
            wind_gust_speed  => $$wcData2{pws}{imperial}{windGust}   ||= 'NA',
            wind_dir         => $$wcData2{airport}{windDirectionCardinal}||= 'NA',
            precip_today     => $$wcData2{airport}{precip24Hour}     ||= 'NA',
            snow             => $$wcData2{airport}{snow24Hour}       ||= 'NA',
            visibility       => $$wcData2{airport}{visibility}       ||= 'NA',
            cloud1h          => $$wcData2{airport}{cloudCeiling}     ||= 'NA',
            cloud1t          => $$wcData2{airport}{cloudCoverPhrase} ||= 'NA',
            condition        => $$wcData2{airport}{wxPhraseLong}     ||= 'NA',
            icon             => $$wcData2{airport}{wxPhraseLong}     ||= 'NA',
        );
    }
    elsif ($wundoption eq 'air') {
        %wxLocal = (
            city             => $$wcData2{location}{city}              ||= 'NA',
            name             => $$wcData2{location}{adminDistrict}     ||= 'NA',
            nameid           => $$wcData2{airport}{displayName}        ||= 'NA',
            latitude         => $$wcData2{location}{latitude}          ||= 'NA',
            longitude        => $$wcData2{location}{longitude}         ||= 'NA',
            elevation        => $$wcData2{airport}{elevation}          ||= 'NA',
            condition        => $$wcData2{airport}{wxPhraseLong}       ||= 'NA',
            icon             => $$wcData2{airport}{wxPhraseLong}       ||= 'NA',
            temperature      => $$wcData2{airport}{temperature}        ||= 'NA',
            dewpoint         => $$wcData2{airport}{temperatureDewPoint}||= 'NA',
            humidity         => $$wcData2{airport}{relativeHumidity}   ||= 'NA',
            pressure         => $$wcData2{airport}{pressureAltimeter}  ||= 'NA',
            precip_today     => $$wcData2{airport}{precip24Hour}       ||= 'NA',
            visibility       => $$wcData2{airport}{visibility}         ||= 'NA',
            snow             => $$wcData2{airport}{snow24Hour}         ||= 'NA',
            wind_speed       => $$wcData2{airport}{windSpeed}          ||= 'NA',
            wind_dir         => $$wcData2{airport}{windDirectionCardinal}||= 'NA',
            wind_dir_degrees => $$wcData2{airport}{windDirection}      ||= 'NA',
            wind_gust_speed  => $$wcData2{airport}{windGust2}          ||= 'NA',
            cloud1h          => $$wcData2{airport}{cloudCeiling}       ||= 'NA',
            cloud1t          => $$wcData2{airport}{cloudCoverPhrase}   ||= 'NA',
            updated          => $$wcData2{airport}{validTimeUtc}       ||= 'NA',
            solarRad         => $$wcData2{pws}{solarRadiation}         ||= 'NA',
        );
    }

    $wxLocal{nameid} = ($wxLocal{nameid} eq 'NA') ? 'wunderground.com' : $wxLocal{nameid};

    if    ( $wxLocal{name} ne 'null' and $wxLocal{city} eq 'null' ) { $wxLocal{city} = $wxLocal{name} }
    elsif ( $wxLocal{name} eq 'null' and $wxLocal{city} ne 'null' ) { $wxLocal{name} = $wxLocal{city} }

    if ( $wxLocal{wind_dir_degrees} or $wxLocal{wind_dir_degrees} == 0  ) {
        $wxLocal{wind_dir} = main::find_wind_dir($wxLocal{wind_dir_degrees});
    }

    if ( not $wxLocal{condition} ) {
        $wxLocal{condition} = ($msky{$wxLocal{cloud1t}}) ? $msky{$wxLocal{cloud1t}} : "Clear" ;
        if ( $wxLocal{icon} eq 'nt_' or $wxLocal{icon} eq '' ) {
            $wxLocal{icon} = $wxLocal{condition}
        }
    }
    # die;

    return \%wxLocal
}

sub weatherWxParser {

    my ( $fileToRead ) = @_ ;
    $$fileToRead =~ tr/\\//d;

    my $jsonW = ${main::get_line_num($fileToRead,'getSunV3CurrentObservationsUrlConfig','getMukPageConfigUrlConfig','str')};
    # msgout($jsonW);

    # my $city = $darkSkyLoc;
    my $city = ${main::get_line_num($fileToRead,'"displayName"','"dstEnd"','str')};

    $city =~ s/"displayName":"//;
    $city =~ s/",//;

    # $jsonW =~ tr/\\//d;
    $jsonW =~ s/^.*?,"data":/{"data":/;
    $jsonW =~ s/,"status":.*$/}/;

    my $wxM = main::jsonParser( $jsonW );
    # msgout($jsonW);

    my %wxLocal = (
        city             => $city ||= 'NA',
        name             => $city ||= 'NA',
        nameid           => 'weather.com',
        latitude         => ($latitude)  ? $latitude  : 'NA',
        longitude        => ($longitude) ? $longitude : 'NA',
        elevation        => ($elevation) ? $elevation : 'NA',
        condition        => $$wxM{data}{wxPhraseLong}          ||= 'NA',
        icon             => $$wxM{data}{wxPhraseLong}          ||= 'NA',
        temperature      => $$wxM{data}{temperature}           ||= 'NA',
        dewpoint         => $$wxM{data}{temperatureDewPoint}   ||= 'NA',
        humidity         => $$wxM{data}{relativeHumidity}      ||= 'NA',
        pressure         => $$wxM{data}{pressureAltimeter}     ||= 'NA',
        visibility       => $$wxM{data}{visibility}            ||= 'NA',
        precip_today     => $$wxM{data}{precip24Hour}          ||= 'NA',
        snow             => $$wxM{data}{snow24Hour}            ||= 'NA',
        wind_dir_degrees => $$wxM{data}{windDirection}         ||= 'NA',
        wind_dir         => $$wxM{data}{windDirectionCardinal} ||= 'NA',
        wind_gust_speed  => $$wxM{data}{windGust}              ||= 'NA',
        wind_speed       => $$wxM{data}{windSpeed}             ||= 'NA',
        updated          => $$wxM{data}{validTimeLocal}        ||= 'NA',
        cloud1h          => $$wxM{data}{cloudCeiling}          ||= 'NA',
        cloud1t          => $$wxM{data}{cloudCoverPhrase}      ||= 'NA',
        solarRad         => 'NA',
    );

    return \%wxLocal

}

sub openwxmapParser {

    my ( $fileToRead ) = @_ ;

    my $city = $darkSkyLoc;

    # msgout($jsonW);
    my $wxM = main::jsonParser( $$fileToRead );

    # msgout($$wxM{current}{weather}[0]{main});

    # die;

    my %wxLocal = (
        city             => $city                                   ||= 'NA',
        name             => $city                                   ||= 'NA',
        nameid           => 'openWeatherMap.com',
        latitude         => $$wxM{lat}                              ||= 'NA',
        longitude        => $$wxM{lon}                              ||= 'NA',
        elevation        => $elevation                              ||= 'NA',
        condition        => $$wxM{current}{weather}[0]{main}        ||= 'NA',
        icon             => $$wxM{current}{weather}[0]{description} ||= 'NA',
        temperature      => $$wxM{current}{temp}                    ||= 'NA',
        dewpoint         => $$wxM{current}{dew_point}               ||= 'NA',
        humidity         => $$wxM{current}{humidity}                ||= 'NA',
        pressure         => $$wxM{current}{pressure}                ||= 'NA',
        # visibility       => $$wxM{current}{visibility}              ||= 'NA',
        precip_today     => $$wxM{current}{precip24Hour}            ||= 'NA',
        snow             => $$wxM{current}{snow24Hour}              ||= 'NA',
        wind_dir_degrees => $$wxM{current}{wind_deg}                ||= 'NA',
        wind_dir         => $$wxM{current}{windDirectionCardinal}   ||= 'NA',
        wind_gust_speed  => $$wxM{current}{windgust}                ||= 'NA',
        wind_speed       => $$wxM{current}{wind_speed}              ||= 'NA',
        updated          => $$wxM{current}{dt}                      ||= 'NA',
        cloud1h          => $$wxM{current}{clouds}                  ||= 'NA',
        cloud1t          => $$wxM{current}{cloudCoverPhrase}        ||= 'NA',
        solarRad         => 'NA',
    );

    $wxLocal{pressure} = ($wxLocal{pressure} ne 'NA') ? main::hPa_in($wxLocal{pressure}) : 'NA' ;
    # $wxLocal{visibility} = ($wxLocal{visibility} ne 'NA') ? $wxLocal{visibility} / 5280 : 'NA' ;

    # my %retH = %$wxM ;

    # return( \%wxLocal, \%retH )
    return( \%wxLocal, \%$wxM )

}


sub yahooWxParser {

    my ( $fileToRead ) = @_ ;

    my $yahooData = ${main::get_line_num( $fileToRead, '"WeatherStore":{','"forecasts"', 'str')};

    $yahooData =~ s/.*"imperial",/{/;
    $yahooData =~ s/,$/}/;

    # msgout($yahooData);
    my $yWx = main::jsonParser( $yahooData );

    my %wxLocal = (
        city             => $$yWx{location}{displayName} ||= 'NA',
        name             => 'Yahoo',
        nameid           => $$yWx{location}{photoWoeid} ||= 'NA',
        latitude         => $$yWx{location}{latitude} ||= 'NA',
        longitude        => $$yWx{location}{longitude} ||= 'NA',
        elevation        => ($elevation) ? $elevation : 'NA',
        condition        => $$yWx{observation}{conditionDescription} ||= 'NA',
        icon             => $$yWx{observation}{conditionDescription} ||= 'NA',
        temperature      => $$yWx{observation}{temperature}{now} ||= 'NA',
        dewpoint         => 'NA',
        humidity         => $$yWx{observation}{humidity} ||= 'NA',
        pressure         => $$yWx{observation}{barometricPressure} ||= 'NA',
        visibility       => $$yWx{observation}{visibility} ||= 'NA',
        precip_today     => 'NA',
        snow             => 'NA',
        wind_speed       => $$yWx{observation}{windSpeed} ||= 'NA',
        wind_dir         => main::find_wind_dir($$yWx{observation}{windDirection}),
        wind_dir_degrees => $$yWx{observation}{windDirection} ||= 'NA',
        wind_gust_speed  => 'NA',
        'updated'        => $$yWx{observation}{observationTime}{timestamp} ||= 'NA',
        cloud1h          => 'NA',
        cloud1t          => 'NA',
        solarRad         => 'NA',
    );

    $wxLocal{updated} =~ s/Z//;

    return \%wxLocal

}

sub timeDateWxParser {

    my ( $fileToRead, $opts ) = @_ ;

    my $y1 = 0;
    my ($wxLocal);
    my $tndData = ${main::get_line_num( $fileToRead, '<section id=bk-focus class=bk-wt>','/section', 'str')};

    $wxLocal = getCurrent($tndData, $wxMainTitle);
    # msgout('kkk',$$wxLocal{updated});
    # $$wxLocal{name} = $$wxLocal{nameid} = $$wxLocal{station};
    if ($opts) {
        $$wxLocal{city} = $$wxLocal{name} = $$wxLocal{station} = $opts
    }

    if ( $$wxLocal{wind_dir_degrees} or $$wxLocal{wind_dir_degrees} == 0  ) {
        $$wxLocal{wind_dir} = main::find_wind_dir($$wxLocal{wind_dir_degrees});
    }

    $$wxLocal{icon} = $$wxLocal{condition};

    $$wxLocal{nameid} = 'timeanddate.com';

    $$wxLocal{updated} =~ s/Z//;


    return \%$wxLocal

}


sub metarWxParser {

    my ( $fileToRead ) = @_ ;

    my %wxLocal;
    my $metarLocal = ${main::get_line_num( $fileToRead, $icao_id,',','str' )};

    if ( $metarLocal ) {
        %wxLocal = main::metar_parser($metarLocal, 'METAR');
        $wxLocal{metar} = $metarLocal;
    }
    else {
        msgout(" \n\nError: :( \nChoose another METAR Station\nThis Station does not appear to be sending METAR reports.\n") ;
        die;
    }
    $wxLocal{updated} = "${monAbb} $wxLocal{updated} $year Z";

    return \%wxLocal

}


sub accuWxParser {

    my ( $fileToRead ) = @_ ;

    my $accustr =
    ${main::get_line_num($fileToRead,'<title>','</title>','str')};
    $accustr .=
    ${main::get_line_num($fileToRead,'<div class="current-conditions-card','<div class="card-button centered','str')};

    $accustr =~ tr/a-zA-Z0-9<_>,.\/: //cd;
    # msgout($accustr);

    my %wxLocal = getCurrent($accustr, $wxMainTitle);
    if ( $wxLocal{wind_dir_degrees} or $wxLocal{wind_dir_degrees} == 0  ) {
        $wxLocal{wind_dir} = main::find_wind_dir($wxLocal{wind_dir_degrees});
    }
    # $wxLocal{icon}    = $wxLocal{condition};
    # $wxLocal{name}    = $wxLocal{city};
    # $wxLocal{nameid}  = "AccuWX";
    # $wxLocal{updated} = $ltime;

    if ( -e "$temppath/accuwarn" ) {
        my ($warnstr, $c);
        {
            open (FILE, "<:crlf", "$temppath/accuwarn") or die "$noFileMsg accuwarn data file $!";
            local $/ = undef;
            $warnstr = <FILE> ;
            close (FILE);
        }
        my $accuwarn = main::get_line_num( \$warnstr,'<div class="info">','<div class="content awx-story">', 'arr');
        for (@{$accuwarn}) {
            if ( $_ =~ m{<h5 .*>(.*?)</h5>} ) {
                $c++;
                $wxLocal{alertdata} .= "Weather alert $c: ".$1.' | '
            }
        }
    }

    return \%wxLocal

}


sub wxParserCurrent {

    ( $temppath, $currdatam, $noFileMsg, $wxMainTitle, $wxFctTitle, $darkSkyLoc, $userLoc, $monAbb, $ltime, $year, $icao_id, $waqidata, $wundoption, $fctBackUrl, $latitude, $longitude, $elevation ) = @_ ;

    my ($wxParser, $wxBack);

    if ( -e $currdatam ) {

        my ( $htmlstr  ) ;
        {
        open (FILE, "<:crlf", $currdatam) or die "$noFileMsg main data file $!";
            local $/ = undef;
            $htmlstr = <FILE> ;
        close (FILE);
        }

        ### wunderground weather ################################################
        if ( $wxMainTitle eq "wund" ) {

            # $wxParser = wundergroundWxParser( \$htmlstr, $wxParser );
            $wxParser = wundergroundWxParser( \$htmlstr );

        }

        ### weather.com weather ################################################
        elsif ( $wxMainTitle eq "wx" ) {

            $wxParser = weatherWxParser( \$htmlstr );

            ## weather forecast
            if ( $wxFctTitle eq "wx" ) {
                %fctM = weatherFctParser( \$htmlstr );
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### darksky.net weather ################################################
        elsif ( $wxMainTitle eq "darkSky" ) {

            $wxParser = darkskyWxParser( \$htmlstr, $darkSkyLoc );

            if ( $wxFctTitle eq "darkSky") {
                %fctM = darkskyFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### yahoo weather ################################################
        elsif ( $wxMainTitle eq "yahoo" ) {

            $wxParser = yahooWxParser( \$htmlstr );

            ### yahoo forecast
            if ( $wxFctTitle eq "yahoo" ) {
                %fctM = yahooFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### timeanddate weather ################################################
        elsif ( $wxMainTitle eq "timeDate" ) {

            $wxParser = timeDateWxParser( \$htmlstr, $darkSkyLoc );

            ### yahoo forecast
            if ( $wxFctTitle eq "timeDate" ) {
                %fctM = timeDateFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### openweathermap ################################################
        elsif ( $wxMainTitle eq "openwxmap" ) {

            my ($ret);

            ($wxParser, $ret) = openwxmapParser( \$htmlstr );
            # $wxParser = openwxmapParser( \$htmlstr );

            # my %retH = $wxM;

            ### openwxmap forecast
            if ( $wxFctTitle eq "openwxmap" ) {
                # %fctM = openwxmapFctParser( \$htmlstr ) ;
                %fctM = openwxmapFctParser( \%$ret ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### msn.com weather ################################################
        elsif ( $wxMainTitle eq "msn" ) {

            $wxParser = msnWxParser( \$htmlstr );

            if ( $wxFctTitle eq "msn" ) {
                %fctM = msnFctParser( \$htmlstr );
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### METAR weather ################################################
        elsif ( $wxMainTitle eq "metar" ) {

            $wxParser = metarWxParser( \$htmlstr );

        }

        ### accuweather.com weather ################################################
        elsif ( $wxMainTitle eq "accu" ) {

            $wxParser = accuWxParser( \$htmlstr );

            ### accu forecast see below
        }
        ### end of wx section


        ### important do not delete - see conkywx.pl file end triggers
        $$wxParser{dataIsThere} = 1 ;

        if ( not $alreadyHaveForecast ) {
            %fctM = wxParserForecast();
            # wxParserForecast();
        }

        my ($fctret, $wxret, $aqistr );

        ### air quality section
        if ( -e $waqidata ) {
            my ( $airstr  ) ;
            {
            open (FILE, "<:crlf", $waqidata) or die "$noFileMsg waqi data file $!";
                local $/ = undef;
                $airstr = <FILE> ;
            close (FILE);
            }
            my $airdata = main::jsonParser( $airstr );

            $$wxParser{o3}   = $$airdata{data}{iaqi}{o3}{v}   ||= 'NA ' ;
            $$wxParser{pm25} = $$airdata{data}{iaqi}{pm25}{v} ||= 'NA ' ;
            $$wxParser{pm10} = $$airdata{data}{iaqi}{pm10}{v} ||= 'NA ' ;
            $$wxParser{no2}  = $$airdata{data}{iaqi}{no2}{v}  ||= 'NA ' ;
            $$wxParser{so2}  = $$airdata{data}{iaqi}{so2}{v}  ||= 'NA ' ;
            $$wxParser{co}   = $$airdata{data}{iaqi}{co}{v}   ||= 'NA ' ;
            $$wxParser{loc}  = $$airdata{data}{city}{name}    ||= 'NA ' ;

        }

        ### current weather + solar data backup section using darkSky
        if ( -e "$temppath/solValueF" ) {
            my ( $altTD  ) ;
            {
            open (FILE, "<:crlf", "$temppath/solValueF" ) or die "$noFileMsg wind data file $!";
                local $/ = undef;
                $altTD = <FILE> ;
            close (FILE);
            }
            $wxBack = weatherWxParser( \$altTD );
            %fctB = weatherFctParser( \$altTD );
        }

        # ### add elements not in the main hash $wxParser
        foreach (keys %$wxBack) {
            my $k1 = $_ unless exists $$wxParser{$_};
            if ($k1) {
                # msgout($k1);
                $$wxParser{$_} = $$wxBack{$_}
            }
        }

        while ( my ($k1, $v1) = each %$wxParser ) {
            if ((not $v1) or ($v1 eq 'NA')) {
                $v1 = $$wxBack{$k1} ;
            }
            $$wxParser{$k1} = ( $v1 ) ? $v1 : 'NA' ;
            # msgout($k1, $v1);
        }

        ### forecast backup section - the fctBack uses wund forecast
        # if (( -e "$temppath/fctBack.html") and ($wxFctTitle ne "wund" )) {
        #     my ( $fbstr  ) ;
        #     {
        #     open (FILE, "<:crlf", "$temppath/fctBack.html") or die "$noFileMsg fctBack data file $!";
        #         local $/ = undef;
        #         $fbstr = <FILE> ;
        #     close (FILE);
        #     }

        #     %fctB = weatherFctParser( \$fbstr, 'yes' );

        # }

        my %fctMM;
        # if ( $wxFctTitle eq "darkSky") {
            foreach my $k (keys %fctB) {
                my $count = 0;
                foreach my $m (@{$fctB{$k}}) {
                    if (not $fctM{$k}[$count] and exists $fctB{$k}[$count]) {
                        $fctM{$k}[$count] = $fctB{$k}[$count]
                    }
                    $count++;
                }
            }
        # }

        foreach my $k (keys %fctM) {
            my $count = 0;
            foreach my $m (@{$fctM{$k}}) {
                if ((not $m) or (index($m, 'NA') != -1)) {
                    my $repHum = $fctB{hum}[$count] ||= 'NA';
                    my $repCor = $fctB{cor}[$count] ||= 'NA';
                    if (index($m, '|NA') != -1) {
                        $fctM{corhum}[$count] = "$fctM{cor}[$count]|$repHum";
                    }
                    elsif (index($m, 'NA|') != -1) {
                        $fctM{corhum}[$count] = "$repCor|$fctM{hum}[$count]";
                    }
                    else {
                        my $replaceValue = $fctB{$k}[$count] ||= 'NA';
                        $fctM{$k}[$count] = $replaceValue;
                    }
                }
                # msgout($k,$fctM{$k}[$count]);
                $count++
            }
        }


        return( \%$wxParser, \%fctM );
    }

        # while ( my ($k1, $v1) = each %wx ) {
        #     msgout("${k1}=>${v1}");
        #     # $wxret .= "${k1}=>${v1}\n"
        # }

}

1;
